#define HEADER 1  /* to indicate that we're inside of a header file */

/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************
**
** MODULE      compare.h                              (c) D.Schrder 96-97
**
** PROJECT
**
** COMPILER    MS Visual C++ 5.0
**
** AUTHOR      Daniel Schrder
**
** ABSTRACT
**
**
** HISTORY     21.6.97 Portierung auf Win32
**             7.9.96 14:31 isBlankLine() und createIndex() behandeln jetzt
**               auch NL als Zeilenendekennzeichen
**             8.9.96 12:17 alle Funktionen benutzen jetzt keine Endekennung
**               mehr, sondern Lngenangaben. Dadurch ist ein Binrvergleichs-
**               modus mglich!
**
***************************************************************************
**************************************************************************/



/* GLOBAL DEFINES ********************************************************/

// mgliche Werte (bitcodiert?) fr properties (Ma der bereinstimmung zweier Zeilen)
#define DIFFERENT   0
#define IDENTICAL   1



/* GLOBAL CONSTANTS **********************************************************/

#ifdef HEADER
extern struct _line {             
  PUCHAR pointer;          
  UINT length;             
  };

extern struct _combined {
  PUCHAR line0;            
  UINT length0;            
  UINT lineno0;            
  PUCHAR line1;
  UINT length1;
  UINT lineno1;
  UINT properties;         
  };

#endif

/* GLOBAL VARIABLES **********************************************************/


/* GLOBAL FUNCTIONS **********************************************************/

/* FUNCTION **************************************************************/

struct _line *createIndex(PUCHAR text, UINT textlength, int mode);

/*-------------------------------------------------------------------------
SPECIFICATION:  Erzeugt ein Feld von Strukturen, in dem jeder Eintrag auf eine
                neue Zeile in 'text' zeigt und die Lnge dieser Zeile enthlt
                Der bentigte Speicher wird
                dynamisch belegt, der Aufrufer ist fr seine Freigabe verant-
                wortlich. "Neue Zeile" wird durch 'mode' bestimmt.

                Ist options.compressline gesetzt, aber nicht options.binmode, dann
                werden alle Zeilen, die ausschlielich whitespace enthalten, ignoriert.

                Das Ende des erzeugten Feldes wird durch eine NULL angezeigt.

Parameters:     text:       Zeiger auf den zu verarbeitenden Speicherbereich
                textlength: Lnge des zu untersuchenden Speicherbereichs
                mode:       Wird an linelength() durchgereicht, Erklrung siehe dort!

Side Effects:   'text' wird verndert; Hauptspeicher wird dynamisch belegt.

Return Value:   Zeiger auf das erzeugte Feld von Strukturen, oder NULL im
                Fehlerfall

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

struct _combined *createCombinedList(struct _line *index0, struct _line *index1, PUINT pcombinedlength);

/*-------------------------------------------------------------------------
SPECIFICATION:  Die eigentliche Vergleichsroutine; erzeugt aus den bergabe-
                parametern ein Feld von struct _combined - Eintrgen. Jeder
                Eintrag im Ergebnisfeld reprsentiert eine anzuzeigende
                Zeile und enthlt einen Zeiger auf eine Zeile aus text0,
                die Lnge der Zeile in text0, einen Zeiger auf eine Zeile aus
                text 1, die Lnge der Zeile aus text1 und ein 'properties'-
                Feld, das derzeit nur angibt, ob die beiden Zeilen identisch
                sind oder nicht.
                Die Funktion belegt dynamisch Hauptspeicher fr die zu erzeugende
                Liste. Der Aufrufer ist fr dessen Freigabe verantwortlich.
                Die resultierende Liste hat hchstens soviele Eintrge wie index0
                plus index1. Sie ist mit einem NULL-Eintrag terminiert.

Parameters:     index0, index1: Zwei Felder von Strukturen, die die Zeilen der
                  zugehrigen Speicherbereiche beschreiben.
                pcombinedlength: Zeiger auf eine Rckgabevariable, in die
                  die Funktion die Lnge (in Eintrgen, ohne den abschlieenden
                  Nulleintrag) der gerade erstellten Liste eintrgt. Der Inhalt
                  der Variablen beim Aufruf ist irrelevant.

Side Effects:   Hauptspeicher wird dynamisch belegt

Return Value:   Zeiger auf die erzeugte Liste

-------------------------------------------------------------------------*/



#undef HEADER  /* header file ends here */
