#define HEADER 1  /* to indicate that we're inside of a header file */

/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************
**
** MODULE       window.h                             (c) D.Schrder 1996,97
**
** PROJECT
**
** COMPILER     IBM C++ 2.1, MS VC++ 6.0
**
** AUTHOR       Daniel Schrder
**
** ABSTRACT     Alle Bildschirmausgaben des Projektes tfc.
**
** Bildschirmlayout:
**
**  C:\CONFIG.SYS C:\CONFIG.BAK Ŀ
**    1 Blablablablabla bla bla bla blabl 101                                  
**    2                                   102                                  
**    3                                                                        
**    4                                                                        
**    5                                                                        
**    6                                                                        
**    7                                                                        
**    8                                                                        
**    9                                                                        
**   10                                                                        
**   11                                                                        
**    -                                                                        
**    -                                                                        
**   12                                                                        
**   13                                                                        
**   14                                                                        
**   15                                                                        
**   16                                                                        
**   17                                                                        
**   18                                                                        
** 
** TOGGLES  Splitting  Line numbers  Diff chars  Tall display   1..9 tab width
**          Case sensitivity  White space compression  Empty lines  Hex mode
**  SPACE next difference   +/- next/prev difference    F1 help    ESC/ENTER quit
**
**
** HISTORY
**
***************************************************************************
**************************************************************************/



/* GLOBAL DEFINES ********************************************************/

// 16 Vordergrundfarben; B=Bright
#define BLACK      0x00
#define BLUE       0x01
#define GREEN      0x02
#define CYAN       0x03
#define RED        0x04
#define MAGENTA    0x05
#define YELLOW     0x06
#define WHITE      0x07

#define GRAY       0x08
#define BBLUE      0x09
#define BGREEN     0x0A
#define BCYAN      0x0B
#define BRED       0x0C
#define BMAGENTA   0x0D
#define BYELLOW    0x0E
#define BWHITE     0x0F

// 16 Hintergrundfarben; BG_ = Background; B=Bright oder Blink
#define BG_BLACK         0x00
#define BG_BLUE          0x10
#define BG_GREEN         0x20
#define BG_CYAN          0x30
#define BG_RED           0x40
#define BG_MAGENTA       0x50
#define BG_YELLOW        0x60
#define BG_WHITE         0x70

#define BG_GRAY          0x80
#define BG_BBLUE         0x90
#define BG_BGREEN        0xA0
#define BG_BCYAN         0xB0
#define BG_BRED          0xC0
#define BG_BMAGENTA      0xD0
#define BG_BYELLOW       0xE0
#define BG_BWHITE        0xF0

// mgliche Werte (bitcodiert) fr content_dirty; werden auch von windows.c bentigt
#define LINENO  1
#define STRINGS 2

// Indizes in colortable
#define MISC             0
#define LOLITE           1
#define HILITE           2
#define MATCHLINE        3
#define MISSLINEMATCH    4
#define MISSLINEMISS     5
#define LINENUMBER       6



/* GLOBAL CONSTANTS **********************************************************/

#ifdef HEADER
extern struct _windows {        
  UINT x0, y0;           
  UINT x1, y1;           
  UINT rows, cols;       
  };

#endif

/* GLOBAL VARIABLES **********************************************************/

extern HANDLE hConsole;

extern char *BasicPage ;

extern char *XPages[] ;

extern char *ColorSetupPages[] ;


/* GLOBAL FUNCTIONS **********************************************************/

/* FUNCTION **************************************************************/

  BOOL initConsole(void);

/*-------------------------------------------------------------------------
SPECIFICATION:  Initialisiert die Textausgabe auf die Konsole. Das (in einer
                globalen Variablen 'hConsole' abgelegte) Handle fr die
                benutzte Konsole wird geholt und gespeichert.
                 
Parameters:     keine

Side Effects:   Globale Variable hConsole wird beschrieben

Return Value:   TRUE:  alles okay
                FALSE: Fehler

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  UINT readColors(PUCHAR inipath, UINT setDefault);

/*-------------------------------------------------------------------------
SPECIFICATION:  Ist setDefault != 0, dann stellt die Funktion lediglich
                Default-Farben ein. Andernfalls versucht sie,
                die Farbeinstellungen aus einer Konfigurations-
                datei zu lesen. Die Funktion versucht zuerst, TFC.INI
                in dem Verzeichnis, auf das 'path' zeigt, zu ffnen.
                Gelingt ihr das nicht, versucht sie ZTW.INI zu ffnen.
                Gelingt das auch nicht, setzt die Funktion default-Farben.
                Andernfalls liest sie aus einem der beiden Files die
                Farbinformation ein und bertrgt sie in die modulglobale
                Variable 'colortable', wobei nicht alle Komponenten von
                'colortable' gesetzt werden.

Parameters:     inipath: Pfad (mit abschlieendem Backslash), in dem die beiden
                  INI-Dateien gesucht werden sollen.
                setDefault: wenn !=0, dann wird nicht versucht, die beiden
                  INI-Dateien zu lesen (dann wird auch path nicht ausgewertet!);
                  es werden nur Defaults eingestellt.

Side Effects:   Dateien werden gelesen; colortable wird geschrieben.

Return Value:   TRUE: Eine der beiden Dateien konnte gelesen werden
                FALSE: keine der beiden Dateien war zu lesen

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  UINT setupColors(PUCHAR inipath, PUCHAR filename, UINT rows);

/*-------------------------------------------------------------------------
SPECIFICATION: Blendet ein Men auf dem Bildschirm ein, das die Einstellung
               der Farben erlaubt, und realisiert diese Einstellung. Das Ergebnis
               wird in die als 'filename' bergebene Datei gespeichert.
               Die aktuell verwendeten Farben werden angepat.

Parameters:    inipath: Pfad (mit einem Backslash abgeschlossen), in dem sich
                 die INI-Datei 'filename' befindet.
               filename: Name der Datei, ohne Pfad, in die das Ergebnis
                 abgespeichert werden soll.Das Format dieser Datei ist
                 anwendungsspezifisch und auer dieser Funktion nur noch
                 'readColors' bekannt.
               rows: Bildschirmlnge

Side Effects:  Benutzer-Interaktion, Schreib- und Lesezugriff auf globales Feld
               'colortable', Anlegen bzw berschreiben einer Datei.

Return Value:  0: Datei erfolgreich geschrieben
               1: Fehler

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

struct _windows displayMask(UINT cols, UINT rows, UINT splitHorizontal, PUCHAR title0, PUCHAR title1);

/*-------------------------------------------------------------------------
SPECIFICATION: Lt den Cursor verschwinden und
               baut die (einzige) Bildschirmmaske auf. Die ntigen Einstell-
               daten werden bergeben. Der Bildschirm teilt sich in zwei Fenster und
               den Menbereich auf. Die beiden Fenster werden mit den
               beiden 'title'-Parametern berschrieben. Falls ntig, werden
               die 'title' gekrzt, und zwar so, da mindestens immer die ersten
               8 und die letzten 15 Zeichen sichtbar bleiben. Nicht mehr
               darstellbare Zeichen werden durch "..." ersetzt.
               Der Bildschirm wird vorher nicht gelscht, sondern komplett
               berschrieben - damit wird ein Flackern bei wiederholten
               Aufrufen vermieden.
               Die Funktion kommt mit allen Bildschirmmodi zurecht, verwendet
               aber grundstzlich nur 80 Spalten. Im Gegensatz dazu wird
               die Anzahl der Zeilen immer voll ausgenutzt.
               Die Koordinaten der beiden durch die Maske gebildeten Fenster
               (inkl. Rahmen!) werden berechnet und in einer Struktur zurckgegeben.
               Die Struktur beinhaltet jeweils die Koordinaten der linken oberen
               Ecke sowie - gemeinsam fr beide Fenster - die Lnge und Breite.

Parameters:    cols: Anzahl der Spalten des momentanen Bildschirmmodus
               rows: Anzahl der Zeilen des momentanen Bildschirmmodus
               splitHorizontal: falls TRUE, wird der Bildschirm horizontal
                 in zwei Fenster unterteilt, andernfalls vertikal
               title0, title1: "berschriften" ber die beiden Fenster.

Side Effects:  Schreiben auf Bildschirm; der vorherige Inhalt geht verloren

Return Value:  Struktur mit den Koordinaten der beiden Fenster

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  void cls(WORD attribute);

/*-------------------------------------------------------------------------
SPECIFICATION: lscht den Bildschirm, dh er wird mit Leerzeichen und dem
               bergebenen Attribut gefllt. Zudem wird der Cursor in's linke obere
               Eck gesetzt.

Parameters:    Attribut, mit dem der Bildschrim berschrieben werden soll.

Side Effects:  Bildschirm wird gelscht

Return Value:  keine

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  void updateWindows(struct _windows windows, struct _combined *plist, UINT posHorizontal, UINT components);

/*-------------------------------------------------------------------------
SPECIFICATION: Fllt die beiden Fenster mit Inhalt. Dabei wird in einer
               Schleife ber die Anzahl verfgbarer Zeilen (2 weniger
               als windows.rows, da dort der Rahmen mitgerechnet ist)
               fr jedes der beiden Fenster pro Zeile die Zeilennummer
               (nur falls in den options nicht unterbunden) und der
               zugehrige String ausgegeben. Dabei werden abhngig von
               plist[Zeile].properties die Strings farblich als "unterschiedlich"
               markiert. Leere Zeilen haben eine Zeilennummer von 0 (Zhlung
               startet sonst bei 1); diese wird als '-' ausgegeben.
               ber das Ende von plist hinausgehende Zeilen werden komplett
               leer ausgegeben. Beim Aufruf ist sicherzustellen, da plist
               *mindestens* noch den abschlieenden Nulleintrag enthlt!
               Die Funktion beschreibt *immer* genau die komplette Breite des Fensters,
               ein explizites Lschen alter Zeilen findet nicht statt (vermeidet
               Flackern).

Parameters:    windows: Enthlt die Koordinaten der beiden Fenster, wobei der
                 Rahmen enthalten ist. Verfgbar sind also nur rows-2 Zeilen und
                 cols-2 Spalten ab x+1, y+1.
               plist: Zeiger auf Feld von Strukturen, die fr jede Zeile die
                 beiden Zeilen und Zeilennummern aus je einer Datei enthalten.
               posHorizontal: Offset in die Zeile (fr rechts/links-scrollen),
                 ab dem der Zeilenstring ausgegeben werden soll.
               components: Nur die hier aufgefhrten Komponenten werden aktualisiert.
                 Mgliche Werte sind:
                 LINENO:  Zeilennummern aktualisieren
                 STRINGS: Zeilen selbst aktualisieren
                 Die Werte sind bitcodiert und knnen ODER-verknpft werden.

Side Effects:  Schreiben auf Bildschirm; Auslesen von colortable und
               options.

Return Value:  keine

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  void pleaseWait(UINT row);

/*-------------------------------------------------------------------------
SPECIFICATION:  gibt in der durch 'row' angegebenen Zeile ab Spalte 1
                wobei die Zhlung bei 0, 0 beginnt)
                eine Meldung aus, die um etwas Geduld bittet.

Parameters:     keine

Side Effects:   Schreiben auf den Bildschirm; das #define WAIT_MESSAGE wird
                benutzt.

Return Value:   keine

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  void PositionIndicator(UINT x, UINT y, UINT width, UCHAR attrindex,
    UINT overall, UINT part, UINT start);

/*-------------------------------------------------------------------------
SPECIFICATION:  Erzeugt einen horizontalen Positionsanzeiger. Die gesamte
                Breite ('width') wird ab Bildschirmposition x,y mit einer
                einfachen Linie in den Farben 'colortable[attrindex]' ge-
                kennzeichnet. Darauf wird eine krzere Doppellinie gelegt,
                die eine Abbildung beispielsweise des Verhltnisses der
                Gesamtlnge einer Datei zum sichtbaren Teil der Datei
                darstellt, und zwar sowohl was die Position als auch die
                Lnge des Ausschnittes angeht.
                Dazu wird eine Bezugsgre 'overall' (Gesamtdateilnge), die
                Lnge des Ausschnittes ('part') und der Beginn des Aus-
                schnittes ('start', von 0 bis 'overall'-'part' - wobei die
                Funktion kulanterweise start <= overall akzeptiert). Aus
                diesen Angaben berechnet sich die Lnge und Position der
                Doppellinie.
                Um ein Flackern zu verhindern, wird die Kombination
                aus Einfach- und Doppellinien in einer Variablen vorbereitet
                und dann gemeinsam ausgegeben.

Parameters:     x, y: Beginn der Anzeige, in VIO-Koordinaten. Die linke obere
                 Ecke ist 0,0.
                width: die Breite der Anzeige, in VIO-Einheiten. Darf nicht
                  grer 132 sein!
                attrindex: index in colortable[], um die Vorder- und Hintergrundfarbe der Anzeige festzulegen
                overall: Basis fr die Berechnung
                part: Lnge des Ausschnittes, 0...overall
                start: Beginn des Ausschnittes, 0...overall - part

Side Effects:   Schreiben auf den Bildschirm; Lesen der modulglobalen Variablen
                colortable

Return Value:   keiner

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

    void printHelp(void);

/*-------------------------------------------------------------------------
SPECIFICATION:  zeigt einen Hilfetext an, falls das Programm ohne oder
                mit falschen Parametern aufgerufen wurde. Der Text wird
                ohne Farbattribute oder Cursorpositionierungen (also quasi
                als TTY) ausgegeben

PARAMETERS:     keine

SIDE EFFECTS:   Ausgabe eines Hilfetextes auf den Bildschirm (stdout)

RETURN VALUES:  keine

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

    void printxHelp(PUCHAR *helptext, UINT rows);

/*-------------------------------------------------------------------------
SPECIFICATION:  zeigt einen erweiterten Hilfetext an. Dabei werden Farb-
                attribute und Cursorpositionierung verwendet.

                Sollte aufgerufen werden, wenn der Benutzer F1 drckt.

PARAMETERS:     helptext: Feld von Hilfe-Strings
                rows:     Seitenlnge

SIDE EFFECTS:   Ausgabe eines Hilfetextes auf den Bildschirm (stdout)

RETURN VALUES:  keine

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  int tprintf(const char *format, ...);

/*-------------------------------------------------------------------------
SPECIFICATION:  Wie printf, tut allerdings nichts, wenn in der globalen
                Variablen "option" das Feld '.ztree' auf TRUE steht.

PARAMETERS:     wie printf

SIDE EFFECTS:   Liest globale Variable 'option'. Sonst wie printf()

RETURN VALUES:  0, wenn option.ztree TRUE ist. Wie printf sonst

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  int GetWindowHeight(void);

/*-------------------------------------------------------------------------
SPECIFICATION:  Gibt die Hhe des momentanen Screenbuffers zurck.

PARAMETERS:     keine

SIDE EFFECTS:   Betriebssystemaufrufe

RETURN VALUES:  Hhe des Fensters in Character-Mode-Zeilen, z.B. 25

-------------------------------------------------------------------------*/


/* FUNCTION **************************************************************/

  int SetWindowHeight(int height);

/*-------------------------------------------------------------------------
SPECIFICATION:  Setzt Fenster und Screenbuffer auf die bergebene Hhe

PARAMETERS:     zu setzende Hhe, in Character-Mode-Zeilen

SIDE EFFECTS:   Betriebssystemaufrufe

RETURN VALUES:  die neue tatschliche Hhe; bei fehlerfreier Ausfhrung
                ist das 'height', andernfalls das, was Windows als
                Fensterhhe zurckliefert.

-------------------------------------------------------------------------*/



#undef HEADER  /* header file ends here */
