/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************

 MODULE      io.c

 PROJECT     TFC for Win32

 COMPILER    Microsoft Visual C++

 DATE        22.6.97

 AUTHOR      D. Schrder

 ABSTRACT    Datei-Ein-/Ausgaberoutinen

 HISTORY     22.6.97 Portierung auf MS VC++ 5.0
             09.08.97 17:21:39 makeFilenames(): CharToOem() ergnzt

***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include "option.h"

/* EXPORT */
#include "disk.h"

/* GLOBAL DEFINES ***********************************************************/
#ifdef HEADER

/* GLOBAL VARIABLES ******************************************************/

/* GLOBAL CONSTANTS ******************************************************/

#endif

/* LOCAL DEFINES *********************************************************/

/* LOCAL VARIABLES *******************************************************/

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/

/* GLOBAL FUNCTIONS *******************************************************/

/* FUNCTION **************************************************************/

  void makeFilenames(PUCHAR file0, PUCHAR file1, int argc, char *argv[])

/*-------------------------------------------------------------------------
SPECIFICATION:   Analysiert die in argc, argv bergebene Kommandozeile und
                 versucht, daraus mindestens einen Dateinamen zu machen.
                 Gelingt dies, ist aber kein zweiter Dateiname angegeben,
                 wird dieser durch ein paar intelligente Vermutungen
                 erzeugt: ist als zweites nur ein Verzeichnis angegeben,
                 wird der erste Dateiname angehngt. Ist gar keine zweite
                 Angabe vorhanden, wird der erste Dateiname mit einer nach
                 .BAK genderten Extension angenommen. Enthlt der zweite
                 Dateiname als erstes oder letztes Zeichen einen '*', dann
                 wird dieser Teil durch den entsprechenden Teil des ersten
                 Dateinamens ersetzt.
                 Die beiden Dateinamen werden im OEM- (ASCII-) Zeichensatz erzeugt.

Parameters:      file0, file1: Platz fr die beiden zu findenden Dateinamen.
                   Die beiden Felder mssen mindestens _MAX_PATH Zeichen lang
                   und mit "" vorbelegt sein.

                 argc: Anzahl der Elemente in argv
                 argv: Feld der Kommandozeilenparameter, wie es vom Betriebs-
                       system kommt.

Side Effects:    zu Testzwecken knnen Dateien geffnet werden. Sie werden
                 allerdings sofort wieder unverndert geschlossen!
                 Die erzeugten Dateinamen werden in file0 und file1 abgelegt.

Return Value:    keiner

-------------------------------------------------------------------------*/
{
  int i, found = 0;                     // found enthlt Anzahl gefundener Dateien
  UCHAR drive0[_MAX_DRIVE], dir0[_MAX_DIR], name0[_MAX_FNAME], ext0[_MAX_EXT],
        drive1[_MAX_DRIVE], dir1[_MAX_DIR], name1[_MAX_FNAME], ext1[_MAX_EXT];
  FILE *stream;

  for(i = 1; i < argc; i++)
    {
    if(argv[i][0] == '-' ||  argv[i][0] == '/')   // Optionen interessieren uns hier nicht
     continue;

	/* DS 21.2.99 kleine Falle: bei Aufrufen der Form tfc "1" "2\" bergibt das OS nicht etwa
	   1 und 2\, sondern 1 und 2", da der Backslash als Escape-Char interpretiert wird.
	   Den Backslash bauen wir uns selber, kein Problem. ber das Anfhrungszeichen mssen wir killen! */
	if(argv[i][strlen(argv[i])-1] == '\"')
	  argv[i][strlen(argv[i])-1] =0;

    if(found == 0)
      {
      // CharToOem(argv[i], argv[i]);   DS 1999-11-28 gehrt da nicht rein - warum habe ich's eingebaut gehabt??
      _splitpath(argv[i], drive0, dir0, name0, ext0);
      _makepath(file0, drive0, dir0, name0, ext0);
      found++;
      }
    else if(found == 1)
      {
      // CharToOem(argv[i], argv[i]); DS 1999-11-28 gehrt da nicht rein - warum habe ich's eingebaut gehabt??
      _splitpath(argv[i], drive1, dir1, name1, ext1);
      found++;
      }
    else
      {
      if(!options.ztree)
        printf("\nToo many file names (%s)\n", argv[i]);
      file0[0] = 0;        // Kennzeichen, da ein Fehler passiert ist
      break;
      }
    }

  /* der zweite Dateiname fehlte komplett; ersten bernehmen, Endung in .BAK
     ndern oder anhngen, falls keine Endung existiert und gengend Platz ist */
  if(found == 1)
    _makepath(file1, drive0, dir0, name0, ".bak");


  /* zwei Dateinamen gefunden; der zweite *knnte* allerdings ein Directory sein,
     also ggf Namensanteil des ersten Dateinamen anhngen. Auerdem kann entweder
     der Name oder die Extension durch ein '*' ersetzt worden sein - dann entsprechenden
     Teil von file1 einkopieren. */
  else if(found == 2)
    {

    if(name1[0] == '*') strcpy(name1, name0);
    if( ext1[1] == '*') strcpy(ext1, ext0);

    _makepath(file1, drive1, dir1, name1, ext1);

    if(NULL == (stream = fopen(file1, "rb")))     // scheint keine Datei gewesen zu sein
      {
      if(file1[strlen(file1)-1] != '\\' && strlen(file1) < _MAX_PATH - 1)
        strcat(file1, "\\");

      if(strlen(file1) + strlen(name0) + strlen(ext0) < _MAX_PATH)
        sprintf(file1, "%s%s%s", file1, name0, ext0);
      }
    else               // file1 scheint wirklich eine existierende Datei gewesen zu sein
      fclose(stream);
    }

  return;
}


/* FUNCTION **************************************************************/

PUCHAR loadFile(PUCHAR filename, PUINT pFilelength)

/*-------------------------------------------------------------------------
SPECIFICATION:  Ldt die ber filename angegebene Datei in den Arbeits-
                speicher. 'filename' wird nicht weiter interpretiert. Die
                Funktion belegt den ntigen Speicher (entspricht der Datei-
                lnge von 'filename') dynamisch. Der Aufrufer ist dafr ver-
                antwortlich, da er wieder freigegeben wird.
                Die Datei wird im Binrmodus geffnet, gelesen und wieder
                geschlossen.

Parameters:     filename:    Name der zu ladenden Datei
                pFilelength: Zeiger auf Variable, in der die Funktion die
                  Lnge der eingelesenen Datei zurckgibt.

Side Effects:   Zugriff auf Massenspeicher; Hauptspeicher wird belegt.

Return Value:   Zeiger auf das Abbild der Datei im Speicher; im Fehlerfall
                (Datei nicht gefunden, kein dynamischer Speicher) wird
                NULL zurckgegeben und kein Hauptspeicher belegt.

                In pFilelength wird die Lnge der eingelesenen Datei zurck-
                gegeben.

-------------------------------------------------------------------------*/
{
  FILE *stream;
  PUCHAR buffer;

  if(NULL == (stream = fopen(filename, "rb")))
    return(NULL);

  *pFilelength = (size_t)_filelength(_fileno(stream));

  buffer = malloc((size_t)*pFilelength);

  if(buffer != NULL)
    fread(buffer, 1, *pFilelength, stream);

  fclose(stream);

  return(buffer);
}
 
