/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************
**
** MODULE       keyboard.c                       (c) D.Schrder 1996
**
** PROJECT
**
** COMPILER     Microsoft Visual C++ 5.0
**
** AUTHOR       Daniel Schrder
**
** ABSTRACT     Tastatur-Zugriffsroutinen und -Vereinbarungen
**
**
** HISTORY      
**              28.2.99 Umbau von _getch() auf ReadConsoleInput()
**              21.6.97 Portierung auf Win32  
**              22.8.96 19:54 erste Version
**
***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#define WIN32_LEAN_AND_MEAN
#include <windows.h>


/* EXPORT */
#include "keyboard.h"

/* GLOBAL DEFINES ********************************************************/
#ifdef HEADER

// ---------------------- Modifier ---------------------------------------
#define K_EXT       0x100
#define K_SHF       0x200
#define K_ALT       0x400
#define K_CTL       0x800


#define ALL_SHIFTS (LEFT_ALT_PRESSED|LEFT_CTRL_PRESSED|RIGHT_ALT_PRESSED|RIGHT_CTRL_PRESSED|SHIFT_PRESSED)
#define EITHER_ALT_PRESSED  (LEFT_ALT_PRESSED|RIGHT_ALT_PRESSED)
#define EITHER_CTRL_PRESSED (LEFT_CTRL_PRESSED|RIGHT_CTRL_PRESSED)


// -------------------------- Verschiedene -------------------------------
#define ESC         0x001b
#define ENTER       0x000d
#define BACKSPACE   0x0008
#define TAB         0x0009

// ------------------------- graue Tasten --------------------------------
#define PAGEUP		0x21+K_EXT
#define PAGEDOWN	0x22+K_EXT
#define END			0x23+K_EXT
#define HOME		0x24+K_EXT
#define LEFT		0x25+K_EXT
#define UP			0x26+K_EXT
#define RIGHT		0x27+K_EXT
#define DOWN		0x28+K_EXT
#define INS			0x2d+K_EXT
#define DEL			0x2e+K_EXT
#define SPACE       ' '


// ----------------------- Funktionstasten -----------------------
#define F1         112+K_EXT
#define F2         113+K_EXT
#define F3         114+K_EXT
#define F4         115+K_EXT
#define F5         116+K_EXT
#define F6         117+K_EXT
#define F7         118+K_EXT
#define F8         119+K_EXT
#define F9         120+K_EXT
#define F10        121+K_EXT

#define SF1			F1 +K_SHF
#define SF2			F2 +K_SHF
#define SF3			F3 +K_SHF
#define SF4			F4 +K_SHF
#define SF5			F5 +K_SHF
#define SF6			F6 +K_SHF
#define SF7			F7 +K_SHF
#define SF8			F8 +K_SHF
#define SF9			F9 +K_SHF
#define SF10		F10+K_SHF

#define CTRLF1		F1 +K_CTL
#define CTRLF2		F2 +K_CTL
#define CTRLF3		F3 +K_CTL
#define CTRLF4		F4 +K_CTL
#define CTRLF5		F5 +K_CTL
#define CTRLF6		F6 +K_CTL
#define CTRLF7		F7 +K_CTL
#define CTRLF8		F8 +K_CTL
#define CTRLF9		F9 +K_CTL
#define CTRLF10		F10+K_CTL

#define ALTF1		F1 +K_ALT
#define ALTF2		F2 +K_ALT
#define ALTF3		F3 +K_ALT
#define ALTF4		F4 +K_ALT
#define ALTF5		F5 +K_ALT
#define ALTF6		F6 +K_ALT
#define ALTF7		F7 +K_ALT
#define ALTF8		F8 +K_ALT
#define ALTF9		F9 +K_ALT
#define ALTF10		F10+K_ALT

// ------------------------- Modifizierte normale Tasten -------------------
#define ALT_A		65+K_ALT
#define ALT_B		66+K_ALT
#define ALT_C		67+K_ALT
#define ALT_D		68+K_ALT
#define ALT_E		69+K_ALT
#define ALT_F		70+K_ALT
#define ALT_G		71+K_ALT
#define ALT_H		72+K_ALT
#define ALT_I		73+K_ALT
#define ALT_J		74+K_ALT
#define ALT_K		75+K_ALT
#define ALT_L		76+K_ALT
#define ALT_M		77+K_ALT
#define ALT_N		78+K_ALT
#define ALT_O		79+K_ALT
#define ALT_P		80+K_ALT
#define ALT_Q		81+K_ALT
#define ALT_R		82+K_ALT
#define ALT_S		83+K_ALT
#define ALT_T		84+K_ALT
#define ALT_U		85+K_ALT
#define ALT_V		86+K_ALT
#define ALT_W		87+K_ALT
#define ALT_X		88+K_ALT
#define ALT_Y		89+K_ALT
#define ALT_Z		90+K_ALT

#define CTRL_A		65+K_CTL
#define CTRL_B		66+K_CTL
#define CTRL_C		67+K_CTL
#define CTRL_D		68+K_CTL
#define CTRL_E		69+K_CTL
#define CTRL_F		70+K_CTL
#define CTRL_G		71+K_CTL
#define CTRL_H		72+K_CTL
#define CTRL_I		73+K_CTL
#define CTRL_J		74+K_CTL
#define CTRL_K		75+K_CTL
#define CTRL_L		76+K_CTL
#define CTRL_M		77+K_CTL
#define CTRL_N		78+K_CTL
#define CTRL_O		79+K_CTL
#define CTRL_P		80+K_CTL
#define CTRL_Q		81+K_CTL
#define CTRL_R		82+K_CTL
#define CTRL_S		83+K_CTL
#define CTRL_T		84+K_CTL
#define CTRL_U		85+K_CTL
#define CTRL_V		86+K_CTL
#define CTRL_W		87+K_CTL
#define CTRL_X		88+K_CTL
#define CTRL_Y		89+K_CTL
#define CTRL_Z		90+K_CTL


typedef struct
{
	UCHAR  vkey;
	UCHAR  scancode;
	UCHAR  ch;
	USHORT shift;
} KEY_RECORD;

typedef union
{
	struct
	{
		UCHAR ch;
		UCHAR shift;
	} key;
	USHORT word;
} ENCODED_KEY;

typedef union
{
	WORD    key;
	BYTE    ch;
} TRANSLATED_KEY;


#endif

/* GLOBAL VARIABLES ******************************************************/

/* GLOBAL CONSTANTS ******************************************************/

/* LOCAL DEFINES ********************************************************/

/* LOCAL VARIABLES *******************************************************/

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/

/* GLOBAL FUNCTIONS ******************************************************/

#if 0

Alte Version, funktioniert unter Win9x mit geladenem keyboard.sys nicht richtig

/* FUNCTION **************************************************************/

  UINT getKey(void)

/*-------------------------------------------------------------------------
SPECIFICATION: Lscht den Tastaturpuffer und
               wartet dann auf einen Tastendruck von stdin und gibt den
               entsprechenden Code zurck. Funktioniert auch fr
               Tasten mit erweiterten Codes wie Funktionstasten, ESC etc.
               Ist im Tastaturpuffer mehr als ein Zeichen, werden diese
               bis auf das letzte verworfen.

Parameters:    keine

Side Effects:  wartet auf Tastatureingabe

Return Value:  Tastaturcode; bei der Weiterverarbeitung sollte auf die
               symbolischen Vereinbarungen zurckgegriffen werden.

-------------------------------------------------------------------------*/
{
  UINT key;

  while(_kbhit())
    getch();

  key = getch();

  if(key == 0x00 || key == 0xE0)
    key |= getch() << 8;

  return(key);
}

#endif


/* FUNCTION **************************************************************/

  UINT getKey(void)

/*-------------------------------------------------------------------------
SPECIFICATION: Lscht den Tastaturpuffer und
               wartet dann auf einen Tastendruck von stdin und gibt den
               entsprechenden Code zurck. Funktioniert auch fr
               Tasten mit erweiterten Codes wie Funktionstasten, ESC etc.
               Ist im Tastaturpuffer mehr als ein Zeichen, werden diese
               bis auf das letzte verworfen.

Parameters:    keine

Side Effects:  wartet auf Tastatureingabe

Return Value:  Tastaturcode; bei der Weiterverarbeitung sollte auf die
               symbolischen Vereinbarungen zurckgegriffen werden.

-------------------------------------------------------------------------*/
{
	static HANDLE console = INVALID_HANDLE_VALUE;
	int  returncode, result;
	TRANSLATED_KEY translated_keys[2];
	INPUT_RECORD ir;
	KEY_RECORD k;
	static BYTE ks[256];

	if(console == INVALID_HANDLE_VALUE)
	  if(INVALID_HANDLE_VALUE == (console = CreateFile("CONIN$", GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0)))
		return(-1);

	do 
	{ 
		if(!(ReadConsoleInput(console, &ir, 1, &returncode) && returncode))
			continue;

		if (ir.EventType != KEY_EVENT || ir.Event.KeyEvent.bKeyDown == FALSE)
			continue;

		k.vkey     = (UCHAR)ir.Event.KeyEvent.wVirtualKeyCode;
		k.scancode = (UCHAR)ir.Event.KeyEvent.wVirtualScanCode;
		k.ch       = ir.Event.KeyEvent.uChar.AsciiChar;
		k.shift    = (USHORT)ir.Event.KeyEvent.dwControlKeyState;


		ks[VK_CAPITAL] = (k.shift & CAPSLOCK_ON)         ? 0x80 : 0x00;
		ks[VK_MENU   ] = (k.shift & EITHER_ALT_PRESSED)  ? 0x80 : 0x00;
		ks[VK_CONTROL] = (k.shift & EITHER_CTRL_PRESSED) ? 0x80 : 0x00;	
		ks[VK_SHIFT  ] = (k.shift & SHIFT_PRESSED)       ? 0x80 : 0x00;

		if ((returncode = ToAscii(k.vkey, k.scancode, ks, &translated_keys[0].key, FALSE)) < 0)
			return(-1);
 
		if (returncode > 0)
		{
			CharToOem(&translated_keys[returncode-1].ch, (char *)&result);
			result &= 0x00FF;
		}

		else
		{
			if (k.ch && k.ch != 0xE0)    result = k.ch;
			else                         result = k.vkey + K_EXT;
		}

		if (k.shift & EITHER_CTRL_PRESSED)
			result += K_CTL;
		if (k.shift & EITHER_ALT_PRESSED)
			result += K_ALT;
		if ((k.shift & SHIFT_PRESSED) && (returncode == 0))  // nicht fr 'normale' Tasten!
			result += K_SHF;

		return(result);

	}
	while(TRUE);

	return(result);
}

