/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************
**
** MODULE      main.c                             (c) D.Schrder 1997
**
** PROJECT     TFC for Win32
**
** COMPILER    Microsoft Visual C++ 5.0
**
** AUTHOR      Daniel Schrder
**
** ABSTRACT    Ein Dateivergleicher, angelehnt an den jfc aus dem XTree-
**             Paket; Hauptmodul
**
**
** HISTORY
**				6.6.99 New: Kommandozeilenschalter "-a xx" setzt Fenster-
**                          hhe auf xx
**              1.3.99 Fix: Umstellung von _getch() auf ReadConsoleInput()
**                     Fix: \" am Ende der Kommandozeile richtig behandelt
**                     New: Tfc beendet sich jetzt bei identischen Dateien
**             21.8.96 20:36 Ideen, Stoffsammlung
**             29.5.97 Portierung auf Win32
**
***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <winbase.h>             // fr Beep()

#include "compare.h"
#include "disk.h"
#include "keyboard.h"
#include "option.h"
#include "window.h"
#include "utility.h"

/* EXPORT */
#include "main.h"

/* GLOBAL DEFINES ********************************************************/

/* GLOBAL VARIABLES ******************************************************/

/* GLOBAL CONSTANTS ******************************************************/

/* LOCAL DEFINES ********************************************************/
#define MAXPOSHORIZONTAL  8192      /* gibt an, wie weit nach rechts gescrollt werden kann; im Gegensatz zum Vertikalscrollen ein fester Wert */

// mgliche Werte (bitcodiert) fr compare_dirty
#define INDEX   1        /* Indextabellen mssen neu berechnet werden (Zeilenlnge hat sich gendert) */
#define COMPARE 2        /* nur Vergleichstabelle mu neu berechnet werden */

/* LOCAL VARIABLES *******************************************************/

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/

/* GLOBAL FUNCTIONS ******************************************************/

/* FUNCTION **************************************************************/

int main(int argc, char *argv[], char *envp[])

/*-------------------------------------------------------------------------
SPECIFICATION: Hauptfunktion; Ablaufsteuerung.
               Das Projekt zerfllt in zwei groe Teilaufgaben:
               - Vergleichsoperation
               - Darstellung
               Dazu kommen einige Nebenaufgaben wie Kommandozeilen- bzw
               Config-File-Auswertung, Dateioperationen etc

Parameters:    bergabeparameter des Betriebssystems

Side Effects:

Return Value:  ffs

-------------------------------------------------------------------------*/
  {
  int result = 0, OriginalWindowHeight, OurWindowHeight;

  UCHAR file0[_MAX_PATH] = "", file1[_MAX_PATH] = "", inipath[_MAX_PATH],
    Oemfile0[_MAX_PATH] = "", Oemfile1[_MAX_PATH] = "";   // Oemfile0, 1 for display *only*!
  PUCHAR text0 = NULL, text1 = NULL;
  UINT textlen0, textlen1, maxPosVertical;  // legt u.a. fest, wie weit nach unten gescrollt werden kann. Ist die Lnge von 'combined' minus ein paar Zeilen, die immer sichtbar bleiben sollen */
  int binlineLength;
  struct _line *index0 = NULL, *index1 = NULL;
  struct _combined *combined = NULL;

  BOOL getOut = FALSE,
    videomode_dirty = FALSE,                // zeigt an, ob der Videomodus neu eingestellt werden mu
    mask_dirty = TRUE,                      // zeigt an, ob die Bildschirmmaske neu aufgebaut werden mu (erster Schleifendurchlauf, Wechsel vert/horiz Teilung); TRUE beinhaltet auch immer ein content_dirty TRUE
    MaskIsOnScreen = FALSE;                 // zeigt an, da die maske schon auf dem Bildschirm ist
  UINT content_dirty = LINENO|STRINGS,      // zeigt an, ob die Anzeige der beiden Dateien aktualisiert werden mu, und zwar sowohl Zeilennummern wie auch die Zeilen selbst
       compare_dirty = INDEX|COMPARE;       // zeigt an, ob die Vergleichsoperationen ausgefhrt werden mssen; wird durch manche Benutzereingaben gesetzt!

  USHORT key;
  UINT posVertical = 0, posHorizontal = 0;  // Anzeige-Koordinaten innerhalb der combined-Liste
  struct _windows windows;


  // Kommandozeilen-Optionen auswerten; bei -? oder Fehler: Hilfe anzeigen, Ende
  getOurPath(argv[0], inipath);
  parseCommandlineOptions(argc, argv);   // Ergebnis wird in globaler Variabler "options" abgelegt

  if(options.bad)
    {
    printHelp();
    return(2);
    }


  // Farbkonfiguration aus Konfigurationsdateien holen
  readColors(inipath, 0);


  // Konsole initialisieren, momentanen Videomode merken, OurWindowHeight initialisieren
  initConsole();

  // OurWindowHeight gem Optionen setzen
  if(options.WindowHeight != 0)
	{
    OurWindowHeight = OriginalWindowHeight = options.WindowHeight;
    videomode_dirty = TRUE;  // fhrt zu einer Neueinstellung der Fensterhhe
	}
  else
    OurWindowHeight = OriginalWindowHeight = GetWindowHeight();


  // Dateinamen aus den bergabeparameter-Variablen extrahieren bzw konstruieren; im Fehlerfall: Meldung, Ende
  makeFilenames(file0, file1, argc, argv);
  CharToOem(file0, Oemfile0);
  CharToOem(file1, Oemfile1);

  if(file0[0] == '\0' || file1[0] == '\0')
	{
	printHelp();
	return(2);
	}


  // ------------------- Fenstertitel setzen --------------------------------
  {
  char title[2*_MAX_PATH + 40];
  sprintf(title, "Tadzio's File Compare %s - %s %s", TFC_VERSION, Oemfile0, Oemfile1);
  SetConsoleTitle(title);
  }

  // ============================== Dateien einlesen ========================
  if(NULL == (text0 = loadFile(file0, &textlen0)))
    {
    tprintf("\nCan't load file 1 (%s)\n", Oemfile0);
    result = 2;
    goto Ende;
    }

  if(NULL == (text1 = loadFile(file1, &textlen1)))
    {
    tprintf("\nCan't load file 2 (%s)\n", Oemfile1);
    result = 2;
    goto Ende;
    }


  // ================== Dateien vergleichen, Datenstrukturen anlegen ========
  
  //Compare files completely (result: 0 = identical, 1 = different)
  if(textlen0 != textlen1)
	result = 1;
  else
	result = memcmp(text0, text1, textlen0)? 1:0;

  // In batch mode or if files are identical: print message
  if(options.batchmode || !result)
	{
    tprintf("\n%s and %s are %sidentical\n\n", Oemfile0, Oemfile1, result? "not ":"");

    if(!options.batchmode)
      {   
      tprintf("\nPress any key to exit!\n");
      getKey();
      }

    goto Ende;
    }

  

  // ----------------------- Schleife ber Benutzereingaben -----------------------------
  do
    {

    // ------------------------ Videomodus setzen -------------------------
    if(videomode_dirty)
      {
      OurWindowHeight = SetWindowHeight(OurWindowHeight);
      mask_dirty = TRUE;                       // neuer Videomodus erzwingt auch immer ein Neuzeichnen der Maske
      videomode_dirty = FALSE;
      }

    // ------------------------ Maske aktualisieren ------------------------
    if(mask_dirty)
      {
      windows = displayMask(80, OurWindowHeight, options.horizontal, Oemfile0, Oemfile1);

      MaskIsOnScreen = TRUE;
      content_dirty = LINENO|STRINGS;                    // Jedes Neuzeichnen der Maske erzwingt ein Neuzeichnen des Inhalts
      mask_dirty = FALSE;
      }


    // ------------------------ Vergleichstabellen erzeugen -----------------------------
    if(compare_dirty)  // es mu neu verglichen werden, mit oder ohne Erzeugung der Indextabellen
      {

      // Zeigerlisten der Speicherabbilder text0, text1 erzeugen. Das ist *in* der Schleife, da binlineLength variable ist!
      // cols-2 ist Fensterbreite OHNE Rahmen; -1 fr Trennspalte; jedes Zeichen der Datei braucht 4 Zeichen auf dem Bildschirm
      
      binlineLength = windows.cols > 72? 16:8;
      if(compare_dirty & INDEX)
        {
        if(index0 != NULL) free(index0);
          index0 = createIndex(text0, textlen0, options.binmode? -binlineLength:1);

        if(index1 != NULL) free(index1);
        index1 = createIndex(text1, textlen1, options.binmode? -binlineLength:1);

        if(index0 == NULL || index1 == NULL)
          {
          tprintf("\nOut of memory\n");
          result = 3;
          goto Ende;
          }
        }

      // Vergleichsoperation: Feld aus Strukturen (zwei Zeiger, zwei Zeilennummern, ein Flag, ob Zeilen gleich oder nicht) belegen
      if(combined != NULL) free(combined);
      combined = createCombinedList(index0, index1, &maxPosVertical);

      if(maxPosVertical > 5) maxPosVertical -=5;
      else                   maxPosVertical = 0;       // Fnf Zeilen sollen immer sichtbar bleiben!
      posVertical = min(posVertical, maxPosVertical);  // durch "Empty Lines" kann maxPosVertical zur Laufzeit kleiner werden!
      content_dirty = LINENO|STRINGS;                  // Jeder neue Vergleichsdurchlauf erzwingt ein Neuzeichnen des Inhaltes der beiden Fenster
      compare_dirty = FALSE;
      }


    // -------------------- Inhalt der Maske (Dateien) aktualisieren ----------
    if(content_dirty)
      {
      updateWindows(windows, combined+posVertical, posHorizontal, content_dirty);

      /* Fenster 0 kriegt nur fr Vertikalteilung einen Indikator, bei
      Horizontalteilung ist die Mittellinie schon vom Dateinamen belegt.
      -4, damit Indikator immer genau ber dem Men landet. */
      if(!options.horizontal)
        PositionIndicator(windows.x0 + 1, OurWindowHeight-4, windows.cols-2, LOLITE,
            maxPosVertical, windows.rows, posVertical);

      PositionIndicator(windows.x1 + 1, OurWindowHeight-4, windows.cols-2, LOLITE,
          maxPosVertical, windows.rows, posVertical);

      content_dirty = FALSE;
      }

  
    // ----------------- Benutzereingabe holen und auswerten ------------------
    switch(key = getKey())
      {
      case 'q':
      case 'Q':
      case ESC:
      case ENTER:             // Ende mit exit code 0
        getOut = TRUE;
        break;

      case F1:
        printxHelp(XPages, OurWindowHeight);
        mask_dirty = TRUE;
        break;

      case UP:
        if(posVertical)
          {
          posVertical--;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case DOWN:
        if(posVertical < maxPosVertical)
          {
          posVertical++;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case LEFT:
        if(posHorizontal > (options.binmode? 0U:1U))
          {
          posHorizontal-= options.binmode? 1:2;
          content_dirty = STRINGS;
          }
        break;

      case RIGHT:
        if(posHorizontal< MAXPOSHORIZONTAL && !options.binmode)  // Rechtsscrollen ist im Binrmodus nicht erlaubt!
          {
          posHorizontal+=options.binmode? 1:2;
          content_dirty = STRINGS;
          }
        break;

      case PAGEUP:
        if(posVertical)
          {
          posVertical -= min(posVertical, windows.rows - 3);
          content_dirty = LINENO|STRINGS;
          }
        break;

      case PAGEDOWN:
        if(posVertical < maxPosVertical)
          {
          posVertical += windows.rows - 3;
          if(posVertical > maxPosVertical) posVertical = maxPosVertical;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case HOME:              // wenn nicht ganz links, dann nach links gehen. *Nur* wenn ganz links: ganz nach oben!
        if(posHorizontal || posVertical)
          {
          if(posHorizontal) posHorizontal = 0;
          else              posVertical = 0;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case END:
        if(posVertical < maxPosVertical)
          {
          posVertical = maxPosVertical;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case SPACE:      // springt bis zum nchsten *Block* mit Unterschieden, daher zuerst raus aus dem aktuellen Block!
      case '+':
        if(posVertical < maxPosVertical)
          {
          while(posVertical < maxPosVertical-1 && combined[posVertical].properties == DIFFERENT)
            posVertical++;

          do
            posVertical++;
          while(posVertical < maxPosVertical && combined[posVertical].properties != DIFFERENT);
          content_dirty = LINENO|STRINGS;
          }
        break;

      case '-':           // springt bis zum vorigen *Block* mit Unterschieden, daher zuerst raus aus dem aktuellen Block!
        if(posVertical > 0)
          {
          while(posVertical > 1 && combined[posVertical].properties == DIFFERENT)
            posVertical--;

          do
            posVertical--;
          while(posVertical > 0 && combined[posVertical].properties != DIFFERENT);
          content_dirty = LINENO|STRINGS;
          }
        break;

      case 't':               // toggle 25/OriginalVideoMode lines mode
      case 'T':
      case ALTF9:
        if(OurWindowHeight!= 25)            OurWindowHeight = 25;
        else if(OriginalWindowHeight == 25) OurWindowHeight = 50;
        else                                OurWindowHeight = OriginalWindowHeight;
        videomode_dirty = TRUE;
        break;

      case 'c':               // Toggle Vergleich ohne/mit Beachtung der Gro-/Kleinschreibung
      case 'C':
        options.caseInsensitive ^=1;
        compare_dirty = COMPARE;
        break;

      case '1':               // Tab-Breite
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        options.tab = key - '0';
        content_dirty = STRINGS;
        break;

      case 'h':                // Hexmodus
      case 'H':
        options.binmode ^= 1;
        if(options.binmode) options.noline = 0;   // Hex-Mode mu *immer* Zeilennummern haben
        posHorizontal &=~1;    // posHorizontal mu beim (Rck-)bergang eine gerade Zahl sein!
        compare_dirty = INDEX|COMPARE;  // Zeilenlnge hat sich gendert
        break;

      case 's':               // Toggle horizontale Teilung
      case 'S':
        options.horizontal ^= 1;
        if(options.binmode)   // Breite hat sich gendert, Zeilenaufteilung neu berechnen!
          compare_dirty = INDEX|COMPARE;  // Zeilenlnge hat sich gendert
                           
        mask_dirty = TRUE;
        break;

      case 'e':               // Toggle Unterdrckung von Leerzeilen
      case 'E':
        options.compressline ^= 1;
        compare_dirty = INDEX|COMPARE;
        break;

      case 'd':               // Toggle Differential Display Mode
      case 'D':
        options.diffmode ^= 1;
        content_dirty = STRINGS;
        break;

      case 'o':               // Toggle OEM Mode
      case 'O':
        options.oem ^= 1;
        content_dirty = STRINGS;
        break;

      case 'l':               // Toggle Zeilennummern ja/nein
      case 'L':
      case 'n':               // rckwrtskompatibel zu JFC
      case 'N':
        if(!options.binmode)
          {
          options.noline ^= 1;
          content_dirty = LINENO|STRINGS;
          }
        break;

      case 'w':               // Toggle Komprimierung von Whitespace
      case 'W':
        options.compresspace ^= 1;
        compare_dirty = COMPARE;
        break;

      case ALTF10:
        setupColors(inipath, "tfc.ini", OurWindowHeight);
        mask_dirty = TRUE;
        break;

      default:
        ; // nichts tun Beep(440, 50);          // Piepsen mit 440Hz fr 50ms
      }
    }
  while(getOut == FALSE);


  // ---------------------- Aufrumen, Ende -------------------------------------
Ende:

  // Videomode zurcksetzen!
  if(OurWindowHeight != OriginalWindowHeight)
    SetWindowHeight(OriginalWindowHeight);

  if(MaskIsOnScreen && !options.ztree)
    cls(WHITE|BG_BLACK);

  if(text0    != NULL) free(text0);
  if(text1    != NULL) free(text1);
  if(index0   != NULL) free(index0);
  if(index1   != NULL) free(index1);
  if(combined != NULL) free(combined);

  return(result);
  }
