/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/


/**************************************************************************
***************************************************************************

 MODULE      option.c

 PROJECT     tfc

 COMPILER    Microsoft Visual C++ 5.0

 DATE        21.6.97

 AUTHOR      D. Schrder

 ABSTRACT    Parser fr Kommandozeilenoptionen

 HISTORY     6.6.99 neue Option fr WindowHeight
			 22.8.96 19:36 erste Version
             20.07.97 13:36:01 Windows-compilierfhige Version

***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "common.h"

/* EXPORT */
#include "option.h"

/* GLOBAL DEFINES ***********************************************************/

/* GLOBAL VARIABLES ******************************************************/
struct _tfc_options options;     // Kommandozeilenoptionen; darf nur von parseCommandlineOptions() beschrieben werden, alle anderen drfen lesen!
                                 // Ausnahme: die Benutzereingabeschleife in main() darf auch schreiben

/* GLOBAL CONSTANTS ******************************************************/
#ifdef HEADER

struct _tfc_options {
  UINT batchmode;
  UINT caseInsensitive;
  UINT tab;
  UINT horizontal;
  UINT compressline;
  UINT noline;
  UINT sync;
  UINT compresspace;
  UINT linecount;
  UINT binmode;
  UINT diffmode;
  UINT ztree;
  UINT bad;
  UINT oem;
  UINT WindowHeight;
  };

#endif

/* LOCAL DEFINES *********************************************************/
#define DEFAULT_TAB 4

/* LOCAL VARIABLES *******************************************************/

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/

/* GLOBAL FUNCTIONS *******************************************************/

/* FUNCTION **************************************************************/

void parseCommandlineOptions(int argc, char *argv[])

/*-------------------------------------------------------------------------
SPECIFICATION:  Analysiert die bergebene Kommandozeile und trgt das Ergebnis
                in die globale Variable options ein. options ist eine
                (projektspezifische) Struktur, die fr alle mglichen
                Kommandozeilenoptionen einen Eintrag enthlt.
                Die Optionen selbst werden *nicht* weiter ausgewertet.
                So wird z.B. bei '-?' keine erweiterte Hilfe angezeigt -
                das bleibt dem Aufrufer berlassen. Einzig bei ungltigen
                Optionen wird ein entsprechender Fehlertext angezeigt.

                Nach Ende der Funktion sind alle Felder in 'options' mit
                gltigen Werten vorbelegt!


Parameters:     argc, argv: die Kommandozeile, wie sie der Lader an main()
                  bergeben hat.

Side Effects:   Schreiben auf globale Variable 'options'

Return Value:   keine

-------------------------------------------------------------------------*/
{
  int i;

  memset(&options, 0, sizeof(options));            // Struktur ausnullen
  options.horizontal = TRUE;					   // Default: Horizontalteilung

  for(i = 1; i < argc; i++)
    {
    if(argv[i][0] != '-' &&  argv[i][0] != '/')   // Optionen mssen mit mit / oder - anfangen
      continue;

    switch(argv[i][1])
      {
	  case 'a':
	  case 'A':
		options.WindowHeight = atoi(argv[i]+2);
		if(options.WindowHeight < 12) options.WindowHeight = 0;
		break;
	  case 'b':
      case 'B': options.batchmode = TRUE; break;
      case 'c':
      case 'C': options.caseInsensitive = TRUE; break;
      case 'd':
      case 'D': options.diffmode = TRUE; break;
      case 'e':
      case 'E': options.compressline = TRUE; break;
      case 'h':
      case 'H': options.binmode = TRUE; break;
      case 'l':
      case 'L':
	  case 'n':
      case 'N': options.noline = TRUE; break;
      case 'o':
      case 'O': options.oem = TRUE; break;
      case 's':
      case 'S': options.sync = TRUE; break;
      case 'v':
      case 'V': options.horizontal = FALSE; break;
      case 'w':
      case 'W': options.compresspace = TRUE; break;
      case 'z':
      case 'Z': options.ztree = TRUE; break;
      
      default:
        if(argv[i][1] >= '1' && argv[i][1] <= '9')
          options.linecount = argv[i][1] - '0';

        else
          {
          printf("\n  Invalid option %s found on the command line\n", argv[i]);
          options.bad = TRUE;
          }
        break;
      }
    }

  options.tab = DEFAULT_TAB;
  if(options.binmode) options.noline = FALSE;

  return;
}
