/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/

/**************************************************************************
***************************************************************************
**
** MODULE       utility.c                             (c) D.Schrder 96,97
**
** PROJECT
**
** COMPILER     Microsoft Visual C++ 5.0
**
** AUTHOR       Daniel Schrder
**
** ABSTRACT     Diverse kleine Hilfsfunktionen
**
**
** HISTORY      21.6.97 Portierung auf Win32
**              22.8.96 23:46
**
***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#include <windows.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

/* EXPORT */
#include "utility.h"

/* GLOBAL DEFINES ********************************************************/

#ifdef HEADER

#define CR '\r'
#define NL '\n'


/* FUNCTION =============================================================*/

#define myisspace(x) ((((x) == ' ') || ((x) == '\t')) ? 1:0) 

/*-------------------------------------------------------------------------
SPECIFICATION: Gibt 1 zurck, wenn das bergebene Zeichen ein Blank oder
               ein Tab war, sonst 0.

Parameters:    x, das untersucht werden soll

Side Effects:  Mehrfache Auswertung von x

Return Value:  0 oder 1

-------------------------------------------------------------------------*/

/* FUNCTION =============================================================*/

#define xtolower(c) _xtolower_table[c]

/*-------------------------------------------------------------------------
SPECIFICATION: Wie das tolower() aus der Bibliothek, aber mit korrekter
               Behandlung der Umlaute

Parameters:    Zeichen, das in einen Kleinbuchstaben gewandelt werden soll

Side Effects:  keine

Return Value:  gewandeltes Zeichen

-------------------------------------------------------------------------*/

/* FUNCTION =============================================================*/

#define hexascii(i) (((i) < 0x0A)? ((i)+'0') : ((i) - 0x0A +'A'))

/*-------------------------------------------------------------------------
SPECIFICATION: Gibt zu einer bergebenen Zahl i, die zwischen 0 und 0x0F
               liegen mu, das ASCII-Zeichen zurck, das diese Zahl in
               Hex darstellt, also '0' fr 0, '1' fr 1, 'A' fr 0x0a etc.
               Fr die Zahlen 0x01 bis 0x0f werden dabei Grobuchstaben
               verwendet.

Parameters:    Ziffer, fr die das ASCII-Zeichen ermittelt werden soll.

Side Effects:  keine

Return Value:  gewandeltes Zeichen

-------------------------------------------------------------------------*/

#endif

/* GLOBAL VARIABLES ******************************************************/
const unsigned int _xtolower_table[256] = {

   0x00,  0x01,  0x02,  0x03,  0x04,  0x05,  0x06,  0x07,  0x08,  0x09,  0x0A,  0x0B,  0x0C,  0x0D,  0x0E,  0x0F,
   0x10,  0x11,  0x12,  0x13,  0x14,  0x15,  0x16,  0x17,  0x18,  0x19,  0x1A,  0x1B,  0x1C,  0x1D,  0x1E,  0x1F,
   0x20,  0x21,  0x22,  0x23,  0x24,  0x25,  0x26,  0x27,  0x28,  0x29,  0x2A,  0x2B,  0x2C,  0x2D,  0x2E,  0x2F,
   0x30,  0x31,  0x32,  0x33,  0x34,  0x35,  0x36,  0x37,  0x38,  0x39,  0x3A,  0x3B,  0x3C,  0x3D,  0x3E,  0x3F,
   0x40,  0x61,  0x62,  0x63,  0x64,  0x65,  0x66,  0x67,  0x68,  0x69,  0x6A,  0x6B,  0x6C,  0x6D,  0x6E,  0x6F,
   0x70,  0x71,  0x72,  0x73,  0x74,  0x75,  0x76,  0x77,  0x78,  0x79,  0x7A,  0x5B,  0x5C,  0x5D,  0x5E,  0x5F,
   0x60,  0x61,  0x62,  0x63,  0x64,  0x65,  0x66,  0x67,  0x68,  0x69,  0x6A,  0x6B,  0x6C,  0x6D,  0x6E,  0x6F,
   0x70,  0x71,  0x72,  0x73,  0x74,  0x75,  0x76,  0x77,  0x78,  0x79,  0x7A,  0x7B,  0x7C,  0x7D,  0x7E,  0x7F,
   0x80,  0x81,  0x82,  0x83,  0x84,  0x85,  0x86,  0x87,  0x88,  0x89,  0x8A,  0x8B,  0x8C,  0x8D,  0x84,  0x8F,
   0x90,  0x91,  0x92,  0x93,  0x94,  0x95,  0x96,  0x97,  0x98,  0x94,  0x81,  0x9B,  0x9C,  0x9D,  0x9E,  0x9F,
   0xA0,  0xA1,  0xA2,  0xA3,  0xA4,  0xA5,  0xA6,  0xA7,  0xA8,  0xA9,  0xAA,  0xAB,  0xAC,  0xAD,  0xAE,  0xAF,
   0xB0,  0xB1,  0xB2,  0xB3,  0xB4,  0xB5,  0xB6,  0xB7,  0xB8,  0xB9,  0xBA,  0xBB,  0xBC,  0xBD,  0xBE,  0xBF,
   0xC0,  0xC1,  0xC2,  0xC3,  0xC4,  0xC5,  0xC6,  0xC7,  0xC8,  0xC9,  0xCA,  0xCB,  0xCC,  0xCD,  0xCE,  0xCF,
   0xD0,  0xD1,  0xD2,  0xD3,  0xD4,  0xD5,  0xD6,  0xD7,  0xD8,  0xD9,  0xDA,  0xDB,  0xDC,  0xDD,  0xDE,  0xDF,
   0xE0,  0xE1,  0xE2,  0xE3,  0xE4,  0xE5,  0xE6,  0xE7,  0xE8,  0xE9,  0xEA,  0xEB,  0xEC,  0xED,  0xEE,  0xEF,
   0xF0,  0xF1,  0xF2,  0xF3,  0xF4,  0xF5,  0xF6,  0xF7,  0xF8,  0xF9,  0xFA,  0xFB,  0xFC,  0xFD,  0xFE,  0xFF
  };

/* GLOBAL CONSTANTS ******************************************************/

/* LOCAL DEFINES ********************************************************/

/* LOCAL VARIABLES *******************************************************/

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/

/* GLOBAL FUNCTIONS ******************************************************/

/* FUNCTION **************************************************************/

  char *filename(char *path)

/*-------------------------------------------------------------------------
SPECIFICATION:  Sucht in dem bergebenen voll-spezifizierten Pfad nach
                dem Dateinamensanteil und gibt einen Zeiger darauf zurck.
                Es wird kein zustzlicher Speicher belegt; der Ergebnis-
                zeiger zeigt immer irgendwo in path hinein.
                Zuerst wird geprft, ob path NULL oder leer ist - in
                diesem Fall wird NULL zurckgegeben.
                Die Funktion sucht rckwrts nach dem ersten Backslash.
                Wenn das nicht das letzte Zeichen ist, wird ein Zeiger
                auf das Folgezeichen zurckgegeben. Ist er das letzte
                Zeichen, dann hat path keinen Namensanteil, es wird NULL
                zurckgegeben. Wird kein Backslash gefunden, dann wird
                vorwrts nach einem Doppelpunkt gesucht; weiteres Verhalten
                wie beim Backslash. Ist auch kein Doppelpunkt vorhanden,
                wird path selbst zurckgegeben.

Parameters:     path: Zeiger auf den zu untersuchenden String.

Side Effects:   keine

Return Value:   oben beschrieben

-------------------------------------------------------------------------*/
{
  char *name;

  if(path == NULL || path[0] == '\0')
    return(NULL);

  name = strrchr(path, '\\');
  if(name == NULL)                  // Backslash nicht gefunden
    name = strchr(path, ':');

  if(name == NULL)                  // Doppelpunkt nicht gefunden
    return(path);

  else
    {
    if((size_t)(++name - path) < strlen(path))
      return(name);
    else
      return(NULL);
    }
}


/* FUNCTION **************************************************************/

void getOurPath(PUCHAR exename, PUCHAR path)

/*-------------------------------------------------------------------------
SPECIFICATION: ermittelt den absoluten Pfad der gerade ausgefhrten .exe-Datei
               und trgt ihn in 'path' ein. 'path' mu dabei mindestens Platz
               fr _MAX_PATH Zeichen haben. 'path' wird immer mit einem
               Backslash abgeschlossen.
               Im Fehlerfall wird path[0] = '\0' gesetzt.

Parameters:    path: Zeiger auf Ziel-Speicherbereich

Side Effects:  keine

Return Value:  Gefundener absoluter Pfad wird nach 'path' kopiert.

-------------------------------------------------------------------------*/
{
  char *p;
  HMODULE hm;
 
  if(NULL == (p = strrchr(exename, '\\')))      // cut off leading path if present
    p = exename;
  else
    p++;

  hm = GetModuleHandle(p); 
  GetModuleFileName(hm, path, _MAX_PATH);

  if(NULL != (p = strrchr(path, '\\')))       // letzten Backslash suchen und...
    p[1] = 0;                                 // Dateinamen abschneiden

  return;
}
                                                                            

/* FUNCTION **************************************************************/

  PUCHAR xstrncpy(PUCHAR target, UINT targetlength, PUCHAR source, signed int sourcelength, UINT tabsize, UINT compresspace)

/*-------------------------------------------------------------------------
SPECIFICATION:  Kopiert bis zu 'targetlength' Zeichen von 'source' nach 'target.
                Ist 'sourcelength' krzer als 'targetlength', wird der Rest mit Leer-
                zeichen aufgefllt. Ist 'sourcelength' grer als 'targetlength', werden
                die berflssigen Zeichen abgeschnitten.
                Jedes '\t' in source wird durch die mittels 'tabsize' angegebene
                Anzahl von Leerzeichen ersetzt. In diesem Fall zhlt der Tab
                nicht als ein, sondern als 'tabsize' Zeichen.
                Ist compresspace != 0, wird jede Folge von mehr als einem Whitespace
                (blank, tab; *nicht* CR oder LF!) zu einem einzelnen Blank zusammengefat
                'target' wird nicht terminiert!

Parameters:     target: Zielspeicherbereich, mu mindestens targetlength Zeichen
                  Platz bieten
                targetlength: Anzahl der nach target zu schreibenden Zeichen.
                source: Quellspeicherbereich; wird in keiner Weise interpretiert,
                  auch nicht evtl enthaltende '\0', CRs oder NLs.
                sourcelength: Lnge des Quellspeicherbereichs. Es werden hchstens
                  so viele Zeichen benutzt wie hiermit angegeben. sourcelength
                  darf auch negativ sein, dann werden gar keine Zeichen daraus
                  benutzt!
                tabsize: Anzahl der Leerzeichen, die fr jeden Tab aus source nach
                  target geschrieben wird.
                compresspace: Wenn dieser Wert ungleich 0 ist, werden alle Folgen von
                  mehr als einem Whitespace (blank, tab) zu einem einzelnen Blank
                  zusammengefat.

Side Effects:   Schreiben nach 'target'

Return Value:   Zeiger auf 'target'

-------------------------------------------------------------------------*/
{
  UINT i = 0, j = 0, k;  // i ist ein Laufindex fr source, j ist ein Laufindex fr target, k einer fr tabsize

  while(j < targetlength)
    {
    if((int)i >= sourcelength)
      target[j++] = ' ';

    else if(compresspace && (i > 0) && myisspace(source[i]) && myisspace(source[i-1]))
      ;                   // nichts tun, zum nchsten i

    else if(source[i] == '\t')
      for(k = 0; k < tabsize && j < targetlength; k++)
        target[j++] = ' ';
    else
      target[j++] = source[i];

    i++;
    }

  return(target);
}

/* FUNCTION **************************************************************/

  PUCHAR hexstrncpy(PUCHAR target, UINT targetlength, PUCHAR source, signed int sourcelength)

/*-------------------------------------------------------------------------
SPECIFICATION:  Kopiert Zeichen von 'source' nach 'target', bis der resultierende
                String in 'target' die Lnge 'targetlength' erreicht hat. Dabei wird
                jedes Zeichen einmal als ASCII-Zeichen, ein-
                mal als Hex-Code dargestellt; die Hex-Codes sind immer zwei
                stellig und stehen (alle zusammen) links von den ASCII-Zeichen.
                Die Hex-Codes sind untereinander mit einem Leerzeichen und
                vom ASCII-Teil mit zwei Leerzeichen getrennt. Die ASCII-
                Zeichen stehen unmittelbar nebeneinander. Dadurch hat der
                resultierende String immer eine "Nutzlnge" von 1 + 4*x
                (wobei x die Anzahl der dargestellten Zeichen ist). Der
                String wird ntigenfalls mit Leerzeichen auf der rechten Seite
                aufgefllt, bis 'targetlength' erreicht ist. Es werden immer so viel
                Zeichen wie mglich dargestellt.
                Enthlt 'source' weniger Zeichen als in 'target' dargestellt
                werden knnten, wird der ASCII-Teil durch Leerzeichen, der
                Hex-Teil durch '--' ersetzt. target wird nicht terminiert!

Parameters:     target: Zielspeicherbereich, mu mindestens targetlength Zeichen
                  Platz bieten
                targetlength: Anzahl der nach target zu schreibenden Zeichen.
                  Die Anzahl der aus source entnommenen Zeichen berechnet sich
                  zu (targetlength-1)/4
                source: Quellspeicherbereich; wird in keiner Weise interpretiert,
                  auch nicht evtl enthaltende '\0', CRs oder NLs.
                sourcelength: Lnge des Quellspeicherbereichs. Es werden hchstens
                  so viele Zeichen benutzt wie hiermit angegeben. sourcelength
                  darf auch negativ sein, dann werden gar keine Zeichen daraus
                  benutzt!

Side Effects:   Schreiben nach 'target'

Return Value:   Zeiger auf 'target'

-------------------------------------------------------------------------*/
{
  int i, chars = (targetlength-1)/4,
    AsciiOffset = 3*chars+1;

  memset(target, ' ', targetlength);

  for(i=0; i < chars; i++)
    {

    if(i >= sourcelength)
      {
      target[3*i+0] = target[3*i+1] = ' ';
      }

    else
      {
      target[3*i+0] = hexascii(source[i]/0x10);
      target[3*i+1] = hexascii(source[i]%0x10);

      target[AsciiOffset + i] = source[i];
      }
    }

  return(target);
}
