/**************************************************************************

    This file is part of 'tfc for windows', a character-mode graphical
	side-by-side file compare utility.

    Copyright (c) 1996 - 2000 by Daniel Schroeder (tadzio@tadzio.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation (see gpl.txt in this package); 
	either version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License (gpl.txt) for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

***************************************************************************/

/**************************************************************************
***************************************************************************
**
** MODULE       window.c                             (c) D.Schrder 1996,97
**
** PROJECT
**
** COMPILER     IBM C++ 2.1, MS-VC++ 6.0
**
** AUTHOR       Daniel Schrder
**
** ABSTRACT     Alle Bildschirmausgaben des Projektes tfc.
**
** Bildschirmlayout:
**
**  C:\CONFIG.SYS C:\CONFIG.BAK Ŀ
**    1 Blablablablabla bla bla bla blabl 101                                  
**    2                                   102                                  
**    3                                                                        
**    4                                                                        
**    5                                                                        
**    6                                                                        
**    7                                                                        
**    8                                                                        
**    9                                                                        
**   10                                                                        
**   11                                                                        
**    -                                                                        
**    -                                                                        
**   12                                                                        
**   13                                                                        
**   14                                                                        
**   15                                                                        
**   16                                                                        
**   17                                                                        
**   18                                                                        
** 
** TOGGLES  Splitting  Line numbers  Diff chars  Tall display   1..9 tab width
**          Case sensitivity  White space compression  Empty lines  Hex mode
**  SPACE next difference   +/- next/prev difference    F1 help    ESC/ENTER quit
**
**
** HISTORY
**
***************************************************************************
**************************************************************************/

/* INCLUDE FILES *********************************************************/

/* IMPORT */
#include <windows.h>
#include <wincon.h>
#include <winbase.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "main.h"                 // fr TFC_VERSION
#include "compare.h"
#include "option.h"
#include "error.h"
#include "utility.h"
#include "keyboard.h"



/* EXPORT */
#include "window.h"

/* GLOBAL DEFINES ********************************************************/

// 16 Vordergrundfarben; B=Bright
#define BLACK      0x00
#define BLUE       0x01
#define GREEN      0x02
#define CYAN       0x03
#define RED        0x04
#define MAGENTA    0x05
#define YELLOW     0x06
#define WHITE      0x07

#define GRAY       0x08
#define BBLUE      0x09
#define BGREEN     0x0A
#define BCYAN      0x0B
#define BRED       0x0C
#define BMAGENTA   0x0D
#define BYELLOW    0x0E
#define BWHITE     0x0F

// 16 Hintergrundfarben; BG_ = Background; B=Bright oder Blink
#define BG_BLACK         0x00
#define BG_BLUE          0x10
#define BG_GREEN         0x20
#define BG_CYAN          0x30
#define BG_RED           0x40
#define BG_MAGENTA       0x50
#define BG_YELLOW        0x60
#define BG_WHITE         0x70

#define BG_GRAY          0x80
#define BG_BBLUE         0x90
#define BG_BGREEN        0xA0
#define BG_BCYAN         0xB0
#define BG_BRED          0xC0
#define BG_BMAGENTA      0xD0
#define BG_BYELLOW       0xE0
#define BG_BWHITE        0xF0

// mgliche Werte (bitcodiert) fr content_dirty; werden auch von windows.c bentigt
#define LINENO  1
#define STRINGS 2

// Indizes in colortable
#define MISC             0
#define LOLITE           1
#define HILITE           2
#define MATCHLINE        3
#define MISSLINEMATCH    4
#define MISSLINEMISS     5
#define LINENUMBER       6

/* GLOBAL VARIABLES ******************************************************/

HANDLE hConsole;

char *BasicPage =
         "\n"
         "             Tadzio's File Compare " TFC_VERSION "\n"
         "             ==========================      (c) 1997/99 Daniel Schroeder\n"
         "                                                    (tadzio@tadzio.com)\n"
         "\n"
         "  command line usage: TFC  [options]  file1  [file2, or a different path]\n"
         "\n"
         "  options:\n"
		 "    -aXX sets screen height to XX lines\n"
		 "    -b  batch mode; no display or input    -o  toggle OEM/ANSI character set\n"
         "    -c  case insensitive compare           -s  sync scan +500 lines\n"
         "    -d  differing chars are highlighted    -v  vertically split display\n"
         "    -e  suppress empty lines               -w  compress white space\n"
         "    -h  hex mode                           -l/-n don't display line numbers\n"
         "\n"
         " 'file1' is the first file to compare. If 'file2' is omitted, 'file1' with its\n"
         "  extension changed to .bak will be used. If 'file2' specifies a directory,\n"
         " 'file1' (without the path portion) will be appended.\n"
         "\n"
         "  In batch mode, TFC returns immediately with the following error levels:\n"
         "    0: files are identical    1: files are not identical\n"
         "    2: file not found         3: out of memory\n"
         "\n";

char *XPages[] = {
     // -------------------------- Page 1 -------------------------------
         "             Tadzio's File Compare "TFC_VERSION"                                        "
         "             ==========================       (c) 1997/99 Daniel Schroeder      "
         "                                                                                "
         "  KEY    DESCRIPTION                                                            "
         "  ---    -----------                                                            "
         "  ESC/ENTER/Q  exit with errorlevel 0                                           "
         "  ARROWS  scroll up, down, left, right, etc                                     "
         "  SPACE   scroll down to next difference                                        "
         "  +       scroll down to next difference                                        "
         "  -       scroll up to previous difference                                      "
         "  C       case sensitivity on/off                                               "
         "  D       toggle highlighting of differing chars within differing lines on/off  "
         "  E       toggle suppression of empty lines                                     "
         "  H       toggle between ASCII and hex mode                                     "
         "  L       toggle line numbers on/off (does not work in hex mode)                "
         "  O       toggle OEM/ANSI character set (OEM is ASCII)                          "
         "  S       toggle vertical/horizontal split screen                               "
         "  T       tall screen toggle (25/50 line mode)                                  "
         "  W       white space compression on/off                                        "
         "  1..9    set tab width to 1..9 blanks (default: 4)                             "
         "  Alt-F10 color setup utility                                                   ",


     // -------------------------- Page 2 -------------------------------
         "                   Specifying file names on the command line                    "
         "                                                                                "
         "  The first file on the command line (file1) can be any valid file name, with or"
         "  without path. Long file names are supported - if they contains blanks,        "
         "  surround them with double quotes. You cannot use wildcards ('*' and '?') here."
         "                                                                                "
         "  The second parameter (file2) can either be a file name or a directory, or you "
         "  may omit it altogether. In this case tfc assumes the same name as file1, with "
         "  the extension changed to '.bak'.                                              "
         "  If you specify a directory, tfc will append the name and extension portion    "
         "  (omitting the drive and directory parts, if any) of file1 to it. You may      "
         "  specify the directory with or without a trailing backslash.                   "
         "  You may replace the name and/or extension portion of file2 with an asterisk.  "
         "  In that case, the corresponding part of file1 is used in place of that        "
         "  wild card. The question mark wild card '?' is not supported.                  "
         "                                                                                ",


     // -------------------------- Page 2a -------------------------------
         "                   Command line examples                                        "
         "                                                                                "
         " This command line               |  compares file     |     to file             "
         " -----------------------------------------------------------------------------  "
         " tfc c:\\config.sys               |  c:\\config.sys     | c:\\config.bak           "
         " tfc c:\\config.sys  c:\\backup    |  c:\\config.sys     | c:\\backup (if it exists)"
         "                                 |                    | c:\\backup\\config.sys    "
         " tfc c:\\config.sys  *.old        |  c:\\config.sys     | c:\\config.old           "
         " tfc c:\\config.sys  d:\\aaa\\*.old |  c:\\config.sys     | d:\\aaa\\config.old       "
         " tfc autoexec.bat   4start.*     |  autoexec.bat      | 4start.bat              ",

     // -------------------------- Page 3 -------------------------------
         "                   Interactive usage                                            "
         "                                                                                "
         "  By default, tfc displays the two file side-by-side on a horizontally split    "
         "  screen. The colors are taken from ZTW.INI, if it can be found in the direc-   "
         "  tory where tfc.exe lives; otherwise, default colors are used. There is, how-  "
         "  ever, a color setup utility built into tfc.                                   "
         "                                                                                "
         "  If the files to compare are identical, tfc tells you so, waits for a key      "
		 "  stroke (not in batch mode) and exits.                                         "
         "                                                                                "
         "  The usual cursor keys can be used to navigate through the two files. They work"
         "  like in any other program, with one additional feature: If you scrolled the   "
         "  display to the right, Pos1 does not jump to the beginning of the files, but   "
         "  rather to the beginning of the line. If you press Pos1 again, it then jumps to"
         "  the beginning of the files.                                                   "
         "                                                                                "
         "  '+' and '-' jump to the next and previous block of different lines,           "
         "  respectively. If there is no such block, they jump to the end resp. beginning "
         "  of the files. The space bar functions identically to '+'.                     "
         "  (continued on next page)                                                      ",

     // -------------------------- Page 4 -------------------------------
         "                   Interactive usage II                                         "
         "                                                                                "
         "  'c' toggles between case sensitive compare (the default) and case-insensitive "
         "  compare. The case-insensitive setting handles German umlauts correctly!       "
         "                                                                                "
         "  'd' toggles highlighting of differing characters within a line on or off.     "
         "  Turning this on may slow down tfc a bit, and may make for a confusing display "
         "  if lines are totally different, so that characters will only randomly match.  "
         "                                                                                "
         "  'e' toggles empty line suppression on and off. If on, all lines that contain  "
         "  nothing but blanks and/or tabs are not displayed.                             "
         "                                                                                "
         "  'h' toggles between hex and ASCII mode. The toggle affects the compare        "
         "  routines as well as the display routines. In hex mode, every line contains    "
         "  either eight (vertical-split) or 16 (horizontal-split) characters.            "
         "  The line number display changes to a file-offset display, using hexadecimal   "
         "  numbers.                                                                      "
         "                                                                                "
         "                                                                                "
         "  (continued on next page)                                                      ",

     // -------------------------- Page 5 -------------------------------
         "                   Interactive usage III                                        "
         "                                                                                "
         "  'l' (thats an ell) toggles line numbers on and off - for those of you who want"
         "  to see more of the files and are not interested in line numbers. The file     "
         "  offset display in hex mode cannot be turned off.                              "
         "                                                                                "
         "  'o' toggles between OEM and ANSI character sets. OEM (or ASCII) is used by    "
         "  DOS-based programs, whereas Windows-based programs use ANSI.                  "
         "                                                                                "
         "  's' toggles between horizontal and vertical splitting of the screen.          "
         "                                                                                "
         "  't' toggles between 25 lines and x lines display, where x is either 50 (if    "
         "  tfc was started in 25-lines-mode) or the number of lines of the active mode   "
         "  when tfc was started (otherwise). Regardless of the state of this toggle,     "
         "  when exiting tfc always switches back to the mode that was active when tfc    "
         "  was started.                                                                  "
         "                                                                                "
         "  'w' toggles white space compression on and off. If on, all sequences of white-"
         "  space characters (blanks, tabs) in the two files are replaced by one single   "
         "  blank. This affects the compare operation as well as the display.             "
         "  (continued on next page)                                                      ",

     // -------------------------- Page 6 -------------------------------
         "                   Interactive usage IV                                         "
         "                                                                                "
         "  '1'...'9' sets the display width of tab characters. Note that this does NOT   "
         "  have any impact on the compare operation, only on the display!                "
         "                                                                                "
         "  Alt-F10 invokes the color setup facility, which has its own help pages.       "
         "  To learn more about it, enter color setup and press F1 there.                 "
         "                                                                                "
         "  ESC, ENTER and 'q' exit tfc.                                                  ",

     // -------------------------- Last Page ----------------------------
         NULL
  };

char *ColorSetupPages[] = {
     // -------------------------- Page 1 -------------------------------
         "                   Color Setup                                                  "
         "                                                                                "
         "  The color setup facility allows you to customize tfc's color scheme.          "
         "  The colors are stored in a file named tfc.ini in the same directory as        "
         "  tfc.exe (which may be different from the current directory!)                  "
         "                                                                                "
         "  The color setup screen shows two sample files which contain elements of all   "
         "  possible colors. Note that during 'real' usage you need to toggle 'Diff chars'"
         "  on to see the 'Differing chars in differing line' color.                      "
         "                                                                                "
         "  To change a foreground color, use uppercase letters, e.g. Shift-f or Shift-i. "
         "  To change background colors, just use the indicated letters, e.g. 'f' or 'i'. "
         "  The meaning of 'f', 'i', and 'l' should be rather self-explanatory.           "
         "  'm' and 'd' are for characters in lines that differ between the two files.    "
         "  'r' restores a built-in default color scheme - in case you got totally        "
         "  lost. 'k' deletes the file 'tfc.ini' from disk. If you called tfc from        "
         "  within ZTree, tfc reverts back to ZTree's color scheme. Otherwise, tfc uses   "
         "                                                                                "
         "  (continued on next page)                                                      ",


     // -------------------------- Page 2 -------------------------------
         "                   Color Setup II                                               "
         "                                                                                "
         "  its default scheme. By pressing ESC, all changes (except deleting tfc.ini)    "
         "  will be discarded. ENTER on the other hand writes the current color scheme to "
         "  tfc.ini, regardless of how the color scheme was created (inherited from ZTree,  "
         "  'R'estored, etc).                                                             "
         "  Note that if you want tfc to inherit ZTree's color scheme each time you use   "
         "  it, you'll have to first 'k'ill tfc.ini, then press ESC - do NOT press ENTER, "
         "  as this would write a new tfc.ini, overriding ZTree's settings.               ",

         NULL };


/* GLOBAL CONSTANTS ******************************************************/
#ifdef HEADER

struct _windows {        // Struktur, die die Koordinaten der beiden Fenster der Maske enthlt
  UINT x0, y0;           // linke obere Ecke des Rahmens des ersten Fensters
  UINT x1, y1;           // linke obere Ecke des Rahmens des zweiten Fensters
  UINT rows, cols;       // Anzahl Zeilen und Spalten, inkl. Rahmen; fr beide Fenster gleich
  };

#endif

/* LOCAL DEFINES ********************************************************/

#define H_TOPLINE      "%1Ŀ"
#define H_LOWERTOPLINE "%1Ĵ"
#define H_MIDLINE      "%1                                                                              "
#define H_BOTLINE      "%1"

#define V_TOPLINE      "%1Ŀ"
#define V_MIDLINE      "%1                                                                             "
#define V_BOTLINE      "%1"

#define MENU "%1 TOGGLES  %2S%1plitting  %2L%1ine numbers  %2D%1iff chars  %2T%1all display  %21..9%1 tab width     " \
             "%1          %2C%1ase sensitivity  %2W%1hite space compression  %2E%1mpty lines  %2O%1EM  %2H%1ex mode " \
             "%1  %2SPACE%1 next difference   %2+/-%1 next/prev difference    %2F1%1 help    %2ESC/ENTER%1 quit "

#define COLORMENUTOP \
              "%1color1.txtcolor2.txtĿ" \
              "%1%6   1 %2Identical line                   %1 %6   1 %2Identical line                   %1" \
              "%1%6   2 %4Line with %5some%4 differences       %1 %6   2 %4Line with %5few %4 differences       %1" \
              "%1%6   3 %5Totally dif%4f%5ering line%4           %1 %6   3 %5Empty line %4f%5ollows    %4           %1" \
              "%1%6   4 %2                                 %1 %6   4 %2                                 %1" \
              "%1%6   5 %2**** Color setup screen ****     %1 %6   5 %2**** Color setup screen ****     %1"

#define COLORMENUMIDDLE \
              "%1%6     %2                                 %1 %6     %2                                 %1"

#define COLORMENUBOTTOM \
              "%1" \
              "%1 COLOR    %2F%1rame and menu   %2I%1dentical lines   %2L%1ine numbers   %2R%1estore defaults    " \
              "%1 SETUP    %2M%1atching chars   %2D%1iffering chars in differing line   %2K%1ill tfc.ini     " \
              "%1  Uppercase: foreground  Lowercase: background   %2F1%1 help   %2ENTER%1 save  %2ESC%1 abort "

#define WAIT_MESSAGE     "Wait, comparing..."

#define FOOTER           "%1   %2PAGEUP%1  previous page      %2PAGEDOWN%1  next page              %2ESC%1  exit help  "

#define SINGLELINE       ""
#define DOUBLELINE       ""

#define MAX_TITLE 132      /* max verfgbarer Speicherplatz fr Titel */
#define MAXSTRING 132      /* max verfgbarer Speicherplatz fr eine Zeile in einem Fenster */


/* LOCAL VARIABLES *******************************************************/

// Farben fr die Anzeige, Defaults siehe readColor(); kann zur Laufzeit gendert werden!
static WORD colortable[7];

/* LOCAL CONSTANTS *******************************************************/

/* LOCAL FUNCTIONS *******************************************************/


/* FUNCTION **************************************************************/

  static void VioWrtCharStrAtt(PUCHAR string, DWORD length, UINT row, UINT col, WORD attr)

/*-------------------------------------------------------------------------
SPECIFICATION:  Schreibt einen String der angegebenen Lnge mit dem
                angegebenen Attribut auf die Console.

Parameters:

Side Effects:

Return Value:   keine

-------------------------------------------------------------------------*/
{
  COORD position;
  DWORD result;

  position.X = col;
  position.Y = row;

  WriteConsoleOutputCharacter(hConsole, string, length, position, &result);
  FillConsoleOutputAttribute(hConsole, attr, length, position, &result);


  return;
}



/* FUNCTION **************************************************************/

  static void showCursor(BOOL visible)

/*-------------------------------------------------------------------------
SPECIFICATION:  Macht den Cursor sichtbar oder unsichtbar, verndert jedoch
                weder Form noch Position.

Parameters:     visible: falls TRUE, wird Cursor sichtbar gemacht
                  falls FALSE, wird Cursor unsichtbar

Side Effects:   Cursor wird sichtbar/unsichtbar

Return Value:   keine

-------------------------------------------------------------------------*/
{
  CONSOLE_CURSOR_INFO CursorData;

  GetConsoleCursorInfo(hConsole, &CursorData);

  CursorData.bVisible = visible;
  SetConsoleCursorInfo(hConsole, &CursorData);

  return;
}


/* FUNCTION **************************************************************/

  static void shortenString(PUCHAR target, PUCHAR source, UINT targetlength)


/*-------------------------------------------------------------------------
SPECIFICATION:  Verkrzt den bergebenen String auf maximal 'targetlength'
                Zeichen. Der source-String bleibt unverndert, das Ergebnis
                wird nach 'target' kopiert. 'target' mu also Platz fr
                mindestens 'targetlength' Zeichen haben (das abschlieende
                Nullzeichen mitgerechnet). Das Ergebnis ist immer null-
                terminiert.
                'targetlength' mu grer 5 sein, andernfalls bleibt
                'target' schlicht unverndert.
                Der Algorithmus: falls source krzer als target ist, wird
                der String einfach umkopiert. Andernfalls werden die ersten
                targetlength/4 Zeichen, dann drei Punkte, dann die
                letzten targetlength - targetlength/4 - 4 Zeichen, dann
                ein Nullzeichen nach 'target' kopiert.

Parameters:     target: Zielspeicherbereich. Mu mindestens Platz fr
                  'targetlength' Zeichen bieten
                source: Quellspeicherbereich. Hieraus werden Zeichen umkopiert,
                  eine Interpretation findet nicht statt.
                targetlength: Lnge von 'target'

Side Effects:   keine

Return Value:   Kein Returncode; das Ergebnis steht im Speicherbereich, auf
                den 'target' zeigt.

-------------------------------------------------------------------------*/
{

  if(targetlength < 5 || target == NULL)
    return;

  if(strlen(source) < targetlength)
    strcpy(target, source);

  else
    {
    strncpy(target, source, targetlength/4);
    strcpy(target+targetlength/4, "...");
    strcat(target, source + strlen(source) - ((targetlength*3)/4 - 4));
    }

  return;
}



/* FUNCTION **************************************************************/

  __inline void tioMakeAttString(PWORD pAttstring, UINT AttstringLength, PUCHAR p0, PUCHAR p1, WORD colorMatch, WORD colorMiss, UINT caseInsensitive)

/*-------------------------------------------------------------------------
SPECIFICATION: Erzeugt einen Attribut-String der Lnge 'AttstringLength'
               aus den beiden Quellstrings p0 und p1. Fr jedes Zeichen
               das sich zwischen p0 und p1 unterscheidet, enthlt pAttstring
               an dieser Stelle den Wert 'colorMiss'. Sind die beiden Zeichen
               gleich (abhngig von caseInsensitive!), erhlt pAttstring an
               dieser Stelle den Wert 'colorMatch'.
               Das Ergebnis wird in dem Speicherbereich aufgebaut, auf den
               pAttstring zeigt. Es hat immer die Lnge AttstringLength, ist
               allerdings nicht terminiert und kann '\0'-Zeichen enthalten

Parameters:    pAttstring: Zeiger auf den Speicherbereich, in dem der Attributstring
                 aufgebaut werden soll. Mu mindestens Platz fr AttstringLength
                 Zeichen bieten.
               AttstringLength: Lnge des aufzubauenden Strings. p0 und p1 mssen
                 mindestens diese Lnge haben!
               p0, p1: Quellstrings, deren Vergleich die Werte fr Attstring liefert.
               colorMatch: Attribut fr zwei bereinstimmende Zeichen
               colorMiss:  Attribut fr zwei nicht bereinstimmende Zeichen

Side Effects:  Speicherbereich, auf den pAttstring zeigt, wird beschrieben

Return Value:  keine

-------------------------------------------------------------------------*/
{
  UINT i;

  if(caseInsensitive)        // Abfrage aus Performancegrnden aus der Schleife gezogen
    {
    for(i=0; i < AttstringLength; i++)
      pAttstring[i] = xtolower(p0[i]) == xtolower(p1[i])? colorMatch : colorMiss;
    }

  else
    {
    for(i=0; i < AttstringLength; i++)
      pAttstring[i] = p0[i] == p1[i]? colorMatch: colorMiss;
    }

  return;
}


/* FUNCTION **************************************************************/

static UINT tioWrtCharStr(PUCHAR string, UINT row, UINT col)

/*-------------------------------------------------------------------------
SPECIFICATION: Gibt einen Textstring auf das Display aus. Die Ausgabe startet
               in Zeile 'row' und Spalte 'col'. Die Attribute werden dem String
               selbst entnommen:
               Das erste Byte nach einem Prozentzeichen wird als Index in eine Attribut-
               tabelle verstanden. Diese Tabelle ist eine (modul-?) globale Variable.
               Alle folgenden Zeichen werden mit diesem Attribut ausgegeben,
               bis wieder ein Prozent-Attributindex-Prchen folgt.
               Der Index 0 wird als Default-Attribut verwendet, bevor das erste
               Prozent-Index-Prchen auftritt.
               Das Prozentzeichen darf nicht als letztes Zeichen des Strings
               auftreten!
               Die Funktion kann derzeit *keine* Prozentzeichen ausgeben!
               Die Funktion benutzt einen statischen Speicher von 16kB Gre.
               Das ist gro genug fr einen kompletten Bildschirm in 80*100-Modus
               incl Attribute. Sollte trotzdem ein zu langer string (ca 8k)
               angegeben werden, bricht die Funktion mit einem Fehlercode ab.
               Achtung, durch den statischen Puffer ist die Funktion *nicht*
               reentrant!

Parameters:    string: der auszugebende String, mit eingestreuten Attribut-Indexangaben
               row: Zeile, in der die Ausgabe beginnen soll. Oberste Zeile ist 0
               col: Spalte, in der die Ausgabe beginnen soll. Linkeste Spalte ist 0

Side Effects: Schreiben auf das Display (stdout).

Return Value:  Wert != 0: string zu lang
               0 sonst

-------------------------------------------------------------------------*/
{
  #define WRITE_BUFFER 8000

  static UCHAR charbuffer[WRITE_BUFFER];
  static WORD attbuffer[WRITE_BUFFER];
  COORD position;
  WORD attribute = colortable[MISC];
  DWORD dummy;  
  int stringlength, i, j=0;                   // j ist Index in buffer

  position.X = col;
  position.Y = row;
  stringlength = strlen(string);              // Aus Performancegrnden in einer Variablen

  for(i=0; i < stringlength; i++)
    {
    if(string[i] == '%')
      attribute = colortable[string[++i] - '0'];
    else
      {
      if(j > WRITE_BUFFER-3) return(1);     // Pufferberlauf
      charbuffer[j] = string[i];
      attbuffer[j++] = attribute;
      }
    }

  WriteConsoleOutputCharacter(hConsole, charbuffer, j, position, &dummy);
  WriteConsoleOutputAttribute(hConsole, attbuffer, j, position, &dummy);

  return(0);
}


/* FUNCTION **************************************************************/

  UINT tioWrtCharAttStr(PUCHAR string, PWORD att, UINT length, UINT row, UINT col)

/*-------------------------------------------------------------------------
SPECIFICATION: Gibt den String 'string' ab der Position 'row' und 'col'
               auf das Display aus, wobei fr jedes Zeichen einzeln die
               Attribute in 'att' festgelegt werden knnen. 'att' mu ebenso
               wie 'string' die Lnge 'length' haben. Irgendwelche Ende-
               kennzeichen werden nicht beachtet oder gefordert.

Parameters:    string: Der auszugebende String. Mu nicht nullterminiert sein,
                 darf '\0` enthalten.
               att: Der zu string gehrige Attribute-String.
               length: Lnge von string und att
               row, col: Position, ab der string ausgegeben werden soll.
                 Die linke obere Bildschirmecke ist 0, 0.

Side Effects:  Schreiben auf den Bildschirm

Return Value:  0: alles okay
               1: kein Speicher

-------------------------------------------------------------------------*/
{
  DWORD dummy;
  COORD position;

  position.X = col;
  position.Y = row;

  WriteConsoleOutputCharacter(hConsole, string, length, position, &dummy);
  WriteConsoleOutputAttribute(hConsole, att,    length, position, &dummy);
  return(0);
    
}


/* GLOBAL FUNCTIONS ******************************************************/

/* FUNCTION **************************************************************/

  BOOL initConsole(void)

/*-------------------------------------------------------------------------
SPECIFICATION:  Initialisiert die Textausgabe auf die Konsole. Das (in einer
                globalen Variablen 'hConsole' abgelegte) Handle fr die
                benutzte Konsole wird geholt und gespeichert.
                 
Parameters:     keine

Side Effects:   Globale Variable hConsole wird beschrieben

Return Value:   TRUE:  alles okay
                FALSE: Fehler

-------------------------------------------------------------------------*/
{
  hConsole = GetStdHandle(STD_OUTPUT_HANDLE);

  return(INVALID_HANDLE_VALUE == hConsole? FALSE:TRUE);
}


/* FUNCTION **************************************************************/

  UINT readColors(PUCHAR inipath, UINT setDefault)

/*-------------------------------------------------------------------------
SPECIFICATION:  Ist setDefault != 0, dann stellt die Funktion lediglich
                Default-Farben ein. Andernfalls versucht sie,
                die Farbeinstellungen aus einer Konfigurations-
                datei zu lesen. Die Funktion versucht zuerst, TFC.INI
                in dem Verzeichnis, auf das 'path' zeigt, zu ffnen.
                Gelingt ihr das nicht, versucht sie ZTW.INI zu ffnen.
                Gelingt das auch nicht, setzt die Funktion default-Farben.
                Andernfalls liest sie aus einem der beiden Files die
                Farbinformation ein und bertrgt sie in die modulglobale
                Variable 'colortable', wobei nicht alle Komponenten von
                'colortable' gesetzt werden.

Parameters:     inipath: Pfad (mit abschlieendem Backslash), in dem die beiden
                  INI-Dateien gesucht werden sollen.
                setDefault: wenn !=0, dann wird nicht versucht, die beiden
                  INI-Dateien zu lesen (dann wird auch path nicht ausgewertet!);
                  es werden nur Defaults eingestellt.

Side Effects:   Dateien werden gelesen; colortable wird geschrieben.

Return Value:   TRUE: Eine der beiden Dateien konnte gelesen werden
                FALSE: keine der beiden Dateien war zu lesen

-------------------------------------------------------------------------*/
{
  UCHAR filename[_MAX_PATH];
  FILE *file;
  struct _ini {
   UCHAR  sig[4];
   USHORT mode;
   USHORT logmode;
   USHORT DateFmt;
   UCHAR  sld1;
   UCHAR  headers;
   UCHAR  TimeFmt;
   UCHAR  f1[19];
   UCHAR  yellow;          // Hilited text (dflt: yellow on Blue)
   UCHAR  cyan;            // Normal text  (dflt: cyan on blue)
   UCHAR  grey;            // Hilite bar   (dflt: black on grey)
   UCHAR  thousep;
   UCHAR  green;           // Global/showall/archive scheme (dflt: green on blue)
   UCHAR  black;           // Disabled menu text (dflt: grey on blue)
    } ini;


  sprintf(filename, "%s%s", inipath, "tfc.ini");

  if(!setDefault)
    {
    if(NULL != (file = fopen(filename, "rb")))
      {
      fread(&colortable, sizeof(colortable), 1, file);  // tfc.ini ist derzeit ein 1-zu-1-Abbild von colortable
      fclose(file);
      return(TRUE);
      }

    // ------------------- ZTW.INI auslesen ------------------------
    sprintf(filename, "%s%s", inipath, "ztw.ini");
    if( NULL != (file = fopen(filename, "rb")))
      {
      fread(&ini, sizeof(ini), 1, file);
      fclose(file);

      colortable[MISC]         = ini.cyan;
      colortable[LOLITE]       = ini.cyan;
      colortable[HILITE]       = ini.yellow;
      colortable[MISSLINEMATCH]= 0x08 ^ ((ini.yellow & 0x0F) | BG_RED);  // nur den Vordergrund mit invertiertem Intensity-Bit bernehmen
      colortable[MISSLINEMISS] = colortable[MISSLINEMATCH] ^ 0x08;       // Intensity-Bit des Vordergrundes nochmal invertieren
      colortable[LINENUMBER]   = ini.black;
      return(TRUE);
      }
    }

  // ------------------- Default-Farben setzen ---------------------
  colortable[MISC]         = BCYAN|BG_BLUE;           // Default
  colortable[LOLITE]       = BCYAN|BG_BLUE;           // Rahmen und Men; "Lolite"
  colortable[HILITE]       = BYELLOW|BG_BLUE;         // Men-Hotkeys; "Hilite"
  colortable[MISSLINEMATCH]= BWHITE|BG_RED;           // unterschiedliche Zeilen, gleiche Zeichen; neu fr tfc
  colortable[MISSLINEMISS] = BYELLOW|BG_RED;          // unterschiedliche Zeilen, unterschiedlichen Zeichen; neu fr tfc
  colortable[LINENUMBER]   = WHITE|BG_BLUE;           // Zeilennummer; neu fr tfc

  return(FALSE);
}

/* FUNCTION **************************************************************/

  UINT setupColors(PUCHAR inipath, PUCHAR filename, UINT rows)

/*-------------------------------------------------------------------------
SPECIFICATION: Blendet ein Men auf dem Bildschirm ein, das die Einstellung
               der Farben erlaubt, und realisiert diese Einstellung. Das Ergebnis
               wird in die als 'filename' bergebene Datei gespeichert.
               Die aktuell verwendeten Farben werden angepat.

Parameters:    inipath: Pfad (mit einem Backslash abgeschlossen), in dem sich
                 die INI-Datei 'filename' befindet.
               filename: Name der Datei, ohne Pfad, in die das Ergebnis
                 abgespeichert werden soll.Das Format dieser Datei ist
                 anwendungsspezifisch und auer dieser Funktion nur noch
                 'readColors' bekannt.
               rows: Bildschirmlnge

Side Effects:  Benutzer-Interaktion, Schreib- und Lesezugriff auf globales Feld
               'colortable', Anlegen bzw berschreiben einer Datei.

Return Value:  0: Datei erfolgreich geschrieben
               1: Fehler

-------------------------------------------------------------------------*/
{
  UINT i, getOut = FALSE;    // gltige Werte: FALSE (weitermachen), 1 (Abbruch), 2 (normales Ende, speichern)
  FILE *file;
  UCHAR _filename[_MAX_PATH];


  sprintf(_filename, "%s%s", inipath, filename);

  do
    {
    tioWrtCharStr(COLORMENUTOP, 0, 0);   // Farbmen, Deckel

   for(i=6; i < rows-4; i++)
      tioWrtCharStr(COLORMENUMIDDLE, i, 0);   // Farbmen, Mitte

    tioWrtCharStr(COLORMENUBOTTOM, i, 0);   // Farbmen, Boden


    switch(getKey())               // Benutzer-Eingabeschleife
      {
      case F1:
        printxHelp(ColorSetupPages, rows);
        break;

      case 'f':
        colortable[LOLITE] = (colortable[LOLITE] + 0x10)%0x100;
        break;

      case 'F':
        colortable[LOLITE] = (colortable[LOLITE] & 0xF0) | ((colortable[LOLITE] + 0x01)%0x10);
        break;

      case 'i':
        colortable[HILITE] = (colortable[HILITE] + 0x10)%0x100;
        break;

      case 'I':
        colortable[HILITE] = (colortable[HILITE] & 0xF0) | ((colortable[HILITE] + 0x01)%0x10);
        break;

      case 'l':
        colortable[LINENUMBER] = (colortable[LINENUMBER] + 0x10)%0x100;
        break;

      case 'L':
        colortable[LINENUMBER] = (colortable[LINENUMBER] & 0xF0) | ((colortable[LINENUMBER] + 0x01)%0x10);
        break;

      case 'm':
        colortable[MISSLINEMATCH] = (colortable[MISSLINEMATCH] + 0x10)%0x100;
        break;

      case 'M':
        colortable[MISSLINEMATCH] = (colortable[MISSLINEMATCH] & 0xF0) | ((colortable[MISSLINEMATCH] + 0x01)%0x10);
        break;

      case 'd':
        colortable[MISSLINEMISS] = (colortable[MISSLINEMISS] + 0x10)%0x100;
        break;

      case 'D':
        colortable[MISSLINEMISS] = (colortable[MISSLINEMISS] & 0xF0) | ((colortable[MISSLINEMISS] + 0x01)%0x10);
        break;

      case 'r':                    // Default-Settings
      case 'R':
        readColors(inipath, 1);
        break;

      case 'k':                    // tfc.ini killen */
        remove(_filename);
        readColors(inipath, 0);    // alle nderungen wieder rckgngig machen
        break;


      case ESC:
        readColors(inipath, 0);    // alle nderungen wieder rckgngig machen
        getOut = 1;
        break;

      case ENTER:                  // Abspeichern
        getOut = 2;
        break;

      default:
        ; // Beep(440L, 50L);          // Piepsen mit 440Hz fr 50ms
      }
    }
  while(getOut == FALSE);


  // ---------------------- neue Farbeinstellung abspeichern --------------
  if(getOut == 2)
    {
    if( NULL != (file = fopen(_filename, "wb")))
      {
      fwrite(colortable, sizeof(colortable), 1, file);
      fclose(file);
      }
    else
      return(1);

    }
  return(0);
}


/* FUNCTION **************************************************************/

struct _windows displayMask(UINT cols, UINT rows, UINT splitHorizontal, PUCHAR title0, PUCHAR title1)

/*-------------------------------------------------------------------------
SPECIFICATION: Lt den Cursor verschwinden und
               baut die (einzige) Bildschirmmaske auf. Die ntigen Einstell-
               daten werden bergeben. Der Bildschirm teilt sich in zwei Fenster und
               den Menbereich auf. Die beiden Fenster werden mit den
               beiden 'title'-Parametern berschrieben. Falls ntig, werden
               die 'title' gekrzt, und zwar so, da mindestens immer die ersten
               8 und die letzten 15 Zeichen sichtbar bleiben. Nicht mehr
               darstellbare Zeichen werden durch "..." ersetzt.
               Der Bildschirm wird vorher nicht gelscht, sondern komplett
               berschrieben - damit wird ein Flackern bei wiederholten
               Aufrufen vermieden.
               Die Funktion kommt mit allen Bildschirmmodi zurecht, verwendet
               aber grundstzlich nur 80 Spalten. Im Gegensatz dazu wird
               die Anzahl der Zeilen immer voll ausgenutzt.
               Die Koordinaten der beiden durch die Maske gebildeten Fenster
               (inkl. Rahmen!) werden berechnet und in einer Struktur zurckgegeben.
               Die Struktur beinhaltet jeweils die Koordinaten der linken oberen
               Ecke sowie - gemeinsam fr beide Fenster - die Lnge und Breite.

Parameters:    cols: Anzahl der Spalten des momentanen Bildschirmmodus
               rows: Anzahl der Zeilen des momentanen Bildschirmmodus
               splitHorizontal: falls TRUE, wird der Bildschirm horizontal
                 in zwei Fenster unterteilt, andernfalls vertikal
               title0, title1: "berschriften" ber die beiden Fenster.

Side Effects:  Schreiben auf Bildschirm; der vorherige Inhalt geht verloren

Return Value:  Struktur mit den Koordinaten der beiden Fenster

-------------------------------------------------------------------------*/
{
  UCHAR short_title0[MAX_TITLE], short_title1[MAX_TITLE];
  struct _windows windows;
  UINT i;

  rows -= 3;        // Aus Sicht der Fenster reduziert das Men einfach die verfgbare Bildschirmlnge

  windows.x0 = 0;
  windows.y0 = 0;
  windows.x1 = splitHorizontal? 0 : cols/2;
  windows.y1 = splitHorizontal? rows/2 : 0;
  windows.rows = splitHorizontal? rows/2 : rows;
  windows.cols = splitHorizontal? cols : cols/2;

  showCursor(FALSE);

  if(splitHorizontal)
    {

    tioWrtCharStr(H_TOPLINE, 0, 0);                 // oberste Zeile

    for(i=1; i < rows - 1; i++)
      tioWrtCharStr(H_MIDLINE, i, 0);               // Fensterrand

    tioWrtCharStr(H_LOWERTOPLINE, windows.y1, 0);   // erste Zeile unteres Fenster
    tioWrtCharStr(H_BOTLINE, rows - 1, 0);          // unterste Fensterzeile
    }

  else
    {
    tioWrtCharStr(V_TOPLINE, 0, 0);                 // oberste Zeile

    for(i=1; i < rows - 1; i++)
      tioWrtCharStr(V_MIDLINE, i, 0);               // Fensterrand

    tioWrtCharStr(V_BOTLINE, rows - 1, 0);          // unterste Fensterzeile
    }

  shortenString(short_title0, title0, min(MAX_TITLE, windows.cols-1));
  shortenString(short_title1, title1, min(MAX_TITLE, windows.cols-1));
  VioWrtCharStrAtt(short_title0, strlen(short_title0), windows.y0, windows.x0+1, colortable[LOLITE]);
  VioWrtCharStrAtt(short_title1, strlen(short_title1), windows.y1, windows.x1+1, colortable[LOLITE]);

  tioWrtCharStr(MENU, rows, 0);                     // Men

  return(windows);
}


/* FUNCTION **************************************************************/

  void cls(WORD attribute)

/*-------------------------------------------------------------------------
SPECIFICATION: lscht den Bildschirm, dh er wird mit Leerzeichen und dem
               bergebenen Attribut gefllt. Zudem wird der Cursor in's linke obere
               Eck gesetzt und sichtbar gemacht.

Parameters:    Attribut, mit dem der Bildschrim berschrieben werden soll.

Side Effects:  Bildschirm wird gelscht

Return Value:  keine

-------------------------------------------------------------------------*/
{
  CONSOLE_SCREEN_BUFFER_INFO Screen;
  COORD Home = { 0, 0};
  DWORD dummy, times;

  GetConsoleScreenBufferInfo(hConsole, &Screen);

  times = Screen.dwSize.X * Screen.dwSize.Y;

  FillConsoleOutputCharacter(hConsole, ' ', times, Home, &dummy);
  FillConsoleOutputAttribute(hConsole, attribute , times, Home, &dummy);

  SetConsoleTextAttribute(hConsole, attribute);
  SetConsoleCursorPosition(hConsole, Home);
  showCursor(TRUE);

  return;
}


/* FUNCTION **************************************************************/

  void updateWindows(struct _windows windows, struct _combined *plist, UINT posHorizontal, UINT components)

/*-------------------------------------------------------------------------
SPECIFICATION: Fllt die beiden Fenster mit Inhalt. Dabei wird in einer
               Schleife ber die Anzahl verfgbarer Zeilen (2 weniger
               als windows.rows, da dort der Rahmen mitgerechnet ist)
               fr jedes der beiden Fenster pro Zeile die Zeilennummer
               (nur falls in den options nicht unterbunden) und der
               zugehrige String ausgegeben. Dabei werden abhngig von
               plist[Zeile].properties die Strings farblich als "unterschiedlich"
               markiert. Leere Zeilen haben eine Zeilennummer von 0 (Zhlung
               startet sonst bei 1); diese wird als '-' ausgegeben.
               ber das Ende von plist hinausgehende Zeilen werden komplett
               leer ausgegeben. Beim Aufruf ist sicherzustellen, da plist
               *mindestens* noch den abschlieenden Nulleintrag enthlt!
               Die Funktion beschreibt *immer* genau die komplette Breite des Fensters,
               ein explizites Lschen alter Zeilen findet nicht statt (vermeidet
               Flackern).

Parameters:    windows: Enthlt die Koordinaten der beiden Fenster, wobei der
                 Rahmen enthalten ist. Verfgbar sind also nur rows-2 Zeilen und
                 cols-2 Spalten ab x+1, y+1.
               plist: Zeiger auf Feld von Strukturen, die fr jede Zeile die
                 beiden Zeilen und Zeilennummern aus je einer Datei enthalten.
               posHorizontal: Offset in die Zeile (fr rechts/links-scrollen),
                 ab dem der Zeilenstring ausgegeben werden soll.
               components: Nur die hier aufgefhrten Komponenten werden aktualisiert.
                 Mgliche Werte sind:
                 LINENO:  Zeilennummern aktualisieren
                 STRINGS: Zeilen selbst aktualisieren
                 Die Werte sind bitcodiert und knnen ODER-verknpft werden.

Side Effects:  Schreiben auf Bildschirm; Auslesen von colortable und
               options.

Return Value:  keine

-------------------------------------------------------------------------*/
{
  UINT i, pastNull = FALSE;
  UCHAR string0[MAXSTRING] = "    ", string1[MAXSTRING] = "    ";
  WORD attstring[MAXSTRING];
  UINT stringcols, dx;
  

  if(!options.binmode)
    {
    dx = options.noline? 0:5;
    stringcols = windows.cols - 2 - dx;
    }
  else
    {
    stringcols = windows.cols > 72? 65:33;
    dx = windows.cols - stringcols-2;
    }


  for(i=0; i < windows.rows-2; i++)
    {
    if(!pastNull)
      if(plist[i].line0 == NULL)
        {
        pastNull = TRUE;            // merken, da wir den Endeintrag erreicht haben!
        memset(string0, ' ', MAXSTRING);
        memset(string1, ' ', MAXSTRING);
        }

    if(pastNull)
      {
      if(components & LINENO)
        {
        VioWrtCharStrAtt(string0, dx, windows.y0+1+i, windows.x0+1, colortable[LINENUMBER]);
        VioWrtCharStrAtt(string1, dx, windows.y1+1+i, windows.x1+1, colortable[LINENUMBER]);
        }

      if(components & STRINGS)
        {
        VioWrtCharStrAtt(string0, stringcols, windows.y0+1+i, windows.x0+1+dx, colortable[HILITE]);
        VioWrtCharStrAtt(string1, stringcols, windows.y1+1+i, windows.x1+1+dx, colortable[HILITE]);
        }
      }

    else
      {
      if(!options.noline && (components & LINENO))
        {
        if(plist[i].lineno0 == 0)
          sprintf(string0, "  -                 ");
        else
          {
          if(options.binmode)
            sprintf(string0, "%0*X ", dx-1, (plist[i].lineno0 - 1) * ((stringcols-1)/4));
          else
            sprintf(string0, "%*i ", dx-1, plist[i].lineno0);
          }
        VioWrtCharStrAtt(string0, dx, windows.y0+1+i, windows.x0+1, colortable[LINENUMBER]);

        if(plist[i].lineno1 == 0)
          sprintf(string1, "  -                 ");
        else
          {
          if(options.binmode)
            sprintf(string1, "%0*X ", dx-1, (plist[i].lineno1 -1) * ((stringcols-1)/4));
          else
            sprintf(string1, "%4i ", plist[i].lineno1);
          }

        VioWrtCharStrAtt(string1, dx, windows.y1+1+i, windows.x1+1, colortable[LINENUMBER]);
        }

      if(components & STRINGS)
        {
        if(options.binmode)
          {
          hexstrncpy(string0, stringcols, plist[i].line0, (signed int)plist[i].length0);
          hexstrncpy(string1, stringcols, plist[i].line1, (signed int)plist[i].length1);
          }
        else
          {
          xstrncpy(string0, stringcols, plist[i].line0 + posHorizontal, (signed int)plist[i].length0 - posHorizontal, options.tab, options.compresspace);
          xstrncpy(string1, stringcols, plist[i].line1 + posHorizontal, (signed int)plist[i].length1 - posHorizontal, options.tab, options.compresspace);
          }

        if(options.oem)         // convert strings to OEM representation
          {
          CharToOem(string0, string0);
          CharToOem(string1, string1);
          }


        if(options.diffmode && plist[i].properties == DIFFERENT)   // nur bei sich unterscheidenden Zeilen ntig
          {
          tioMakeAttString(attstring, stringcols, string0, string1, colortable[MISSLINEMATCH], colortable[MISSLINEMISS], options.caseInsensitive);  // Attribut-String berechnen
          tioWrtCharAttStr(string0, attstring, stringcols, windows.y0+1+i, windows.x0+1+dx);
          tioWrtCharAttStr(string1, attstring, stringcols, windows.y1+1+i, windows.x1+1+dx);
          }
        else
          {
          VioWrtCharStrAtt(string0, stringcols, windows.y0+1+i, windows.x0+1+dx, colortable[plist[i].properties == DIFFERENT? MISSLINEMISS : HILITE]);
          VioWrtCharStrAtt(string1, stringcols, windows.y1+1+i, windows.x1+1+dx, colortable[plist[i].properties == DIFFERENT? MISSLINEMISS : HILITE]);
          }
        }
      }
    }

  return;
}

/* FUNCTION **************************************************************/

  void pleaseWait(UINT row)

/*-------------------------------------------------------------------------
SPECIFICATION:  gibt in der durch 'row' angegebenen Zeile ab Spalte 1
                wobei die Zhlung bei 0, 0 beginnt)
                eine Meldung aus, die um etwas Geduld bittet.

Parameters:     keine

Side Effects:   Schreiben auf den Bildschirm; das #define WAIT_MESSAGE wird
                benutzt.

Return Value:   keine

-------------------------------------------------------------------------*/
{
  VioWrtCharStrAtt(WAIT_MESSAGE, strlen(WAIT_MESSAGE), row, 1, colortable[LOLITE]);
  return;
}


/* FUNCTION **************************************************************/

  void PositionIndicator(UINT x, UINT y, UINT width, UCHAR attrindex,
    UINT overall, UINT part, UINT start)

/*-------------------------------------------------------------------------
SPECIFICATION:  Erzeugt einen horizontalen Positionsanzeiger. Die gesamte
                Breite ('width') wird ab Bildschirmposition x,y mit einer
                einfachen Linie in den Farben 'colortable[attrindex]' ge-
                kennzeichnet. Darauf wird eine krzere Doppellinie gelegt,
                die eine Abbildung beispielsweise des Verhltnisses der
                Gesamtlnge einer Datei zum sichtbaren Teil der Datei
                darstellt, und zwar sowohl was die Position als auch die
                Lnge des Ausschnittes angeht.
                Dazu wird eine Bezugsgre 'overall' (Gesamtdateilnge), die
                Lnge des Ausschnittes ('part') und der Beginn des Aus-
                schnittes ('start', von 0 bis 'overall'-'part' - wobei die
                Funktion kulanterweise start <= overall akzeptiert). Aus
                diesen Angaben berechnet sich die Lnge und Position der
                Doppellinie.
                Um ein Flackern zu verhindern, wird die Kombination
                aus Einfach- und Doppellinien in einer Variablen vorbereitet
                und dann gemeinsam ausgegeben.

Parameters:     x, y: Beginn der Anzeige, in VIO-Koordinaten. Die linke obere
                 Ecke ist 0,0.
                width: die Breite der Anzeige, in VIO-Einheiten. Darf nicht
                  grer 132 sein!
                attrindex: index in colortable[], um die Vorder- und Hintergrundfarbe der Anzeige festzulegen
                overall: Basis fr die Berechnung
                part: Lnge des Ausschnittes, 0...overall
                start: Beginn des Ausschnittes, 0...overall - part

Side Effects:   Schreiben auf den Bildschirm; Lesen der modulglobalen Variablen
                colortable

Return Value:   keiner

-------------------------------------------------------------------------*/
{
  UCHAR buffer[132];
  UINT x1, x2, x3;     // x1 ist der Beginn der Doppellinie, x2 ihre Lnge und x3 der Lnge des Restes

  if(overall == 0) overall = 1;
  if(part > 4*overall/5)     part = 4*overall/5;       // sieht einfach besser aus!
  if(part == 0) part = 1;
  if(start > overall - part) start = overall - part;

  x1 = (start * width)/overall;
  x2 = ( part * width)/overall; if(x2 == 0) x2 = 1;   // die Doppellinie mu mindestens ein Zeichen breit sichtbar sein!
  x3 = width - x2 - x1;

  sprintf(buffer, "%.*s%.*s%.*s", x1, SINGLELINE, x2, DOUBLELINE, x3, SINGLELINE);

  VioWrtCharStrAtt(buffer, strlen(buffer), y, x, colortable[attrindex]);

  return;
}


/* FUNCTION **************************************************************/

    void printHelp(void)

/*-------------------------------------------------------------------------
SPECIFICATION:  zeigt einen Hilfetext an, falls das Programm ohne oder
                mit falschen Parametern aufgerufen wurde. Der Text wird
                ohne Farbattribute oder Cursorpositionierungen (also quasi
                als TTY) ausgegeben

PARAMETERS:     keine

SIDE EFFECTS:   Ausgabe eines Hilfetextes auf den Bildschirm (stdout)

RETURN VALUES:  keine

-------------------------------------------------------------------------*/
{
  tprintf(BasicPage);
  return;
}

/* FUNCTION **************************************************************/

    void printxHelp(PUCHAR *helptext, UINT rows)

/*-------------------------------------------------------------------------
SPECIFICATION:  zeigt einen erweiterten Hilfetext an. Dabei werden Farb-
                attribute und Cursorpositionierung verwendet.

                Sollte aufgerufen werden, wenn der Benutzer F1 drckt.

PARAMETERS:     helptext: Feld von Hilfe-Strings
                rows:     Seitenlnge

SIDE EFFECTS:   Ausgabe eines Hilfetextes auf den Bildschirm (stdout)

RETURN VALUES:  keine

-------------------------------------------------------------------------*/
{
  UCHAR buffer[96];
  int page = 0;

  while(TRUE)
    {
    cls(colortable[LOLITE]);
    sprintf(buffer, "  Page %i                                                                          ", page);
    VioWrtCharStrAtt(buffer, 80, 0, 0, colortable[LOLITE]);                               // Header
    VioWrtCharStrAtt(helptext[page], strlen(helptext[page]), 1, 0, colortable[HILITE]);  // Text
    VioWrtCharStrAtt(SINGLELINE, 80, rows-2, 0, colortable[LOLITE]);                     // Trennlinie
    tioWrtCharStr(FOOTER, rows-1, 0);                                                    // Footer

    // Taste holen und auswerten
    switch(getKey())
      {
      case PAGEUP:
      case UP:
      case LEFT:
        if(page > 0) page--;
        break;

      case HOME:
        page = 0;
        break;

      case F1:
      case PAGEDOWN:
      case DOWN:
      case RIGHT:
      case ENTER:
      case ' ':
        if(helptext[page+1] != NULL) page++;
        break;

      case ESC:
        return;
        break;
      }
    }

  return;
}

/* FUNCTION **************************************************************/

  int tprintf(const char *format, ...)

/*-------------------------------------------------------------------------
SPECIFICATION:  Wie printf, tut allerdings nichts, wenn in der globalen
                Variablen "option" das Feld '.ztree' auf TRUE steht.

PARAMETERS:     wie printf

SIDE EFFECTS:   Liest globale Variable 'option'. Sonst wie printf()

RETURN VALUES:  0, wenn option.ztree TRUE ist. Wie printf sonst

-------------------------------------------------------------------------*/
{
  int result = 0;

  if(!options.ztree)
    {
    va_list argptr;

    va_start(argptr, format);

    result = vprintf(format, argptr);

    va_end(argptr);

    }
  return(result);
}




/* FUNCTION **************************************************************/

  int GetWindowHeight(void)

/*-------------------------------------------------------------------------
SPECIFICATION:  Gibt die Hhe des momentanen Screenbuffers zurck.

PARAMETERS:     keine

SIDE EFFECTS:   Betriebssystemaufrufe

RETURN VALUES:  Hhe des Fensters in Character-Mode-Zeilen, z.B. 25

-------------------------------------------------------------------------*/
{
  CONSOLE_SCREEN_BUFFER_INFO VideoMode;

  GetConsoleScreenBufferInfo(hConsole, &VideoMode);

  // printf("\nWindow Height (Bottom - Top)  is %i\n", VideoMode.srWindow.Bottom - VideoMode.srWindow.Top + 1);
  // printf("\nBuffer Height (dwSize.Y)  is %i\n", VideoMode.dwSize.Y);
  // getchar();


  return(VideoMode.dwSize.Y);
  // return(VideoMode.srWindow.Bottom - VideoMode.srWindow.Top + 1);

}

/* FUNCTION **************************************************************/

  int SetWindowHeight(int Height)

/*-------------------------------------------------------------------------
SPECIFICATION:  Setzt Fenster und Screenbuffer auf die bergebene Hhe

PARAMETERS:     zu setzende Hhe, in Character-Mode-Zeilen

SIDE EFFECTS:   Betriebssystemaufrufe

RETURN VALUES:  die neue tatschliche Hhe; bei fehlerfreier Ausfhrung
                ist das 'height', andernfalls das, was Windows als
                Fensterhhe zurckliefert.

-------------------------------------------------------------------------*/
{
  CONSOLE_SCREEN_BUFFER_INFO VideoMode;
  int oldHeight;

  GetConsoleScreenBufferInfo(hConsole, &VideoMode);
  oldHeight = VideoMode.dwSize.Y;

  VideoMode.dwSize.Y = Height;
  
  VideoMode.srWindow.Top = 0;
  VideoMode.srWindow.Bottom = Height-1;

  if(Height > oldHeight)
    {
    SetConsoleScreenBufferSize(hConsole, VideoMode.dwSize);     // Buffer vergrern
    SetConsoleWindowInfo(hConsole, TRUE, &VideoMode.srWindow);   // Fenster vergrern

    }
  else
    {
    SetConsoleWindowInfo(hConsole, TRUE, &VideoMode.srWindow);  // Fenster verkleinern
    SetConsoleScreenBufferSize(hConsole, VideoMode.dwSize);     // Buffer verkleinern
    }


  // get now-current screen buffer height and return it
  GetConsoleScreenBufferInfo(hConsole, &VideoMode);
  return(VideoMode.dwSize.Y);
}
